#========================================================================================
# package      DE_EPAGES::Amazon::t::GetFeedSubmissionResultResponseHandlerTest
# base         Test::Class
# state        private
#----------------------------------------------------------------------------------------
# description  Tests for GetFeedSubmissionResultResponseHandler
#========================================================================================
package DE_EPAGES::Amazon::t::GetFeedSubmissionResultResponseHandlerTest;
use base Test::Class;

use DE_EPAGES::Amazon::API::MWS::GetFeedSubmissionResultResponseHandler;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use Test::More;
use strict;

# setup methods are run before every test method.
sub setUp : Test(setup) {
    my $self = shift;
    $self->{Handler} = DE_EPAGES::Amazon::API::MWS::GetFeedSubmissionResultResponseHandler->new();
}

sub handleSubmissionResultDocument : Test {
    my $self = shift;
    my $xml = <<'EOL';
<?xml version="1.0" encoding="UTF-8"?>
<AmazonEnvelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="amzn-envelope.xsd">
<Header>
<DocumentVersion>1.02</DocumentVersion>
<MerchantIdentifier>M_EPAGESSOFT_7322224</MerchantIdentifier>
</Header>
<MessageType>ProcessingReport</MessageType>
<Message>
<MessageID>1</MessageID>
<ProcessingReport>
<DocumentTransactionID>2529194756</DocumentTransactionID>
<StatusCode>Complete</StatusCode>
<ProcessingSummary>
<MessagesProcessed>1</MessagesProcessed>
<MessagesSuccessful>1</MessagesSuccessful>
<MessagesWithError>0</MessagesWithError>
<MessagesWithWarning>0</MessagesWithWarning>
</ProcessingSummary>
</ProcessingReport>
</Message>
</AmazonEnvelope>
EOL
    $self->{Handler}->handle($xml);
    is(scalar @{$self->{Handler}->Result()}, 1, 'saved ProcessingResult');
}

sub handleSubmissionError : Test {
    my $self = shift;
    my $xml = <<'EOL';
<?xml version="1.0"?>
<ErrorResponse xmlns="http://mws.amazonaws.com/doc/2009-01-01/">
<Error>
<Type>Sender</Type>
<Code>InvalidRequest</Code>
<Message>INVALID_ID failed a validation check: INVALID_ID is not a valid identifer.</Message>
<Detail/></Error><RequestID>e769694b-c6a1-4850-b419-ede77d3b8087</RequestID></ErrorResponse>
EOL
    eval {
        $self->{Handler}->handle($xml);
        fail('should throw exception');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'InvalidRequest', 'InvalidRequest exception');
    }
}

package main;                                                                     ## no critic (ProhibitMultiplePackages)
use DE_EPAGES::Core::API::Script qw ( RunScript );

RunScript(
    Sub => sub {
        Test::Class->runtests;
    }
);

1;