#========================================================================================
# package      DE_EPAGES::Amazon::t::GetOrderServiceStatusResponseHandlerTest
# base         Test::Class
# state        private
#----------------------------------------------------------------------------------------
# description  Tests for GetOrderServiceStatusResponseHandler
#========================================================================================
package DE_EPAGES::Amazon::t::GetOrderServiceStatusResponseHandlerTest;
use base Test::Class;

use DE_EPAGES::Amazon::API::MWS::GetOrderServiceStatusResponseHandler;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use Test::More;
use strict;

# setup methods are run before every test method.
sub setUp : Test(setup) {
    my $self = shift;
    $self->{Handler} = DE_EPAGES::Amazon::API::MWS::GetOrderServiceStatusResponseHandler->new();
}

sub handleError : Test {
    my $self = shift;
    my $xml = <<'EOL';
<?xml version="1.0"?>
<ErrorResponse xmlns="https://mws.amazonservices.com/Orders/2011-01-01">
<Error>
  <Type>Sender</Type>
  <Code>InvalidRequest</Code>
  <Message>parameter AWSAccessKeyId failed a validation check: value cannot be empty.</Message>
  <Detail/>
</Error>
<RequestID>c3cb7eea-d631-4551-bcea-dc7ae92f9cdb</RequestID>
</ErrorResponse>
EOL
    eval {
        $self->{Handler}->handle($xml);
        fail('should throw exception.');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'InvalidRequest', 'Caught InvalidRequest exception.');
    }
}

package main;                                                                     ## no critic (ProhibitMultiplePackages)
use DE_EPAGES::Core::API::Script qw ( RunScript );

RunScript(
    Sub => sub {
        Test::Class->runtests;
    }
);

1;