#========================================================================================
# package      DE_EPAGES::Amazon::t::RemoteAdapterFactoryTest
# base         Test::Class
# state        private
#----------------------------------------------------------------------------------------
# description  tests the RemoteAdapterFactory
#========================================================================================
package DE_EPAGES::Amazon::t::RemoteAdapterFactoryTest;
use base Test::Class;

use Test::More;
use File::Basename;
use Readonly;
use strict;
use DE_EPAGES::Amazon::API::RemoteAdapterFactory;
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByPath LoadRootObject LoadObjectByPath JoinObjectPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
Readonly my $ROOTALIAS => 'Test';
Readonly my $ROOTPATH  => '/' . $ROOTALIAS;

sub cleanup : Test(shutdown) {
    my $self = shift;
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
}

sub init : Test(startup) {
    my $self = shift;
    return if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    $self->cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTALIAS } );
    ImportObjects( dirname(__FILE__) . '/import.xml', $ROOTPATH );
    $self->{Shop} = LoadObjectByPath( JoinObjectPath( [ '', $ROOTALIAS, 'TestShopAmazon' ] ) );
    $self->{AmazonConfig} = $self->{Shop}->path('PortalConfigs/AmazonTestConfig');
    $self->{AmazonConfig}
      ->set( { 'MWSMerchantID' => 'abc', 'PortalMerchantID' => 'TestingAmazon', 'MerchantLogin' => 'login', 'MerchantPassword' => 'Password' } );

}

sub testgetAmazonConfig : Test {
    my $self = shift;
    isa_ok( $self->{AmazonConfig}, 'DE_EPAGES::Amazon::API::Object::AmazonConfig', 'AmazonConfig object' );
}

sub testNewRemoteAmazonMerchant : Test {
    my $self    = shift;
    my $Adapter = DE_EPAGES::Amazon::API::RemoteAdapterFactory::NewRemoteAmazonMerchant( $self->{AmazonConfig} );
    isa_ok( $Adapter, 'DE_EPAGES::Amazon::API::Object::RemoteAmazonMerchant', 'Adapter object' );
}

sub testNewRemoteMerchantToMWSAdapter : Test {
    my $self    = shift;
    my $Adapter = DE_EPAGES::Amazon::API::RemoteAdapterFactory::NewRemoteMerchantToMWSAdapter( $self->{AmazonConfig} );
    isa_ok( $Adapter, 'DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter', 'Adapter object' );
}

sub testNewRemoteMWS : Test(3) {
    my $self = shift;
    my $MWS  = DE_EPAGES::Amazon::API::RemoteAdapterFactory::NewRemoteMWS( $self->{AmazonConfig} );
    isa_ok( $MWS, 'DE_EPAGES::Amazon::API::Object::RemoteMWS', 'MWS object' );
    is( $MWS->marketplaceID(), 'A1PA6795UKMFR9', 'marketplaceID existant' );
    is( $MWS->merchantID(),    'abc',            'merchantID existant.' );
}

####### Main ############
package main;    ## no critic (ProhibitMultiplePackages)

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );

RunScript(
    Sub => sub {
        RunOnStore(
            'Store' => GetTestSite(),
            'Sub'   => sub {
                Test::Class->runtests;
            }
        );
    }
);

1;
