#========================================================================================
# package      DE_EPAGES::Amazon::t::RemoteMWSTest
# base         Test::Class
# state        private
#----------------------------------------------------------------------------------------
# description  Tests for RemoteMWS adapter (interation tests)
#========================================================================================
package DE_EPAGES::Amazon::t::RemoteMWSTest;
use base Test::Class;

use DE_EPAGES::Amazon::API::Object::RemoteMWS;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use Test::More;
use strict;

# setup methods are run before every test method.
sub setUp : Test(setup) {
    my $self = shift;
    $self->{Remote} = DE_EPAGES::Amazon::API::Object::RemoteMWS->new(
        AWSURL         => 'https://mws.amazonservices.de/',
        AWSAccessKeyId => 'AKIAI3Q3XUVKQLJAF4VQ',
        AWSSecretKeyId => 'ZVqYiipCYjdJ22kShxYZs8hovjVgKyHQ4h+6b2Qr',
        MerchantID     => 'A1LP59BH07J3H8',
        MarketplaceID  => 'A1PA6795UKMFR9',
    );
}

sub getReport_with_invalid_merchant_id : Test(no_plan) {
    my $self = shift;
    $self->{Remote}->{merchantID} = 'INVALID';
    my $Ret = $self->{Remote}->getReport( ReportId => 1, );
    like( $Ret->decoded_content(), qr/InvalidParameterValue/, 'Access using invalid merchant id returns InvalidParameterValue error' );
}

sub getReport_with_invalid_report_id : Test(no_plan) {
    my $self = shift;
    my $Ret = $self->{Remote}->getReport( ReportId => 1, );
    like( $Ret->decoded_content(), qr/InvalidReportId/, 'Access to ReportId=1 returns InvalidReportId error' );
}

sub getReport_without_reportid : Test(no_plan) {
    my $self = shift;
    eval {
        $self->{Remote}->getReport();
        fail('should throw PARAMETER_MISSING');
    };
    if ( ExistsError() ) {
        is( GetError->code, 'PARAMETER_MISSING', 'caught PARAMETER_MISSING exception.' );
    }
}

sub getReportCount : Test {
    my $self = shift;
    my $Ret  = $self->{Remote}->getReportCount();
    like( $Ret->decoded_content(), qr{<Count>\d+</Count>}, 'GetReportCount returns a number' );
}

sub getReportList : Test {
    my $self = shift;
    my $Ret  = $self->{Remote}->getReportList();
    like( $Ret->decoded_content(), qr{<HasNext>\w+</HasNext>}, 'GetReportList returns HasNext' );
}

sub getReportListByNextToken : Test {
    my $self = shift;
    my $Ret  = $self->{Remote}->getReportList();
    if ( $Ret->decoded_content() =~ qr{<NextToken>([^<]+)</NextToken>}x ) {
        $Ret = $self->{Remote}->getReportListByNextToken( NextToken => $1 );
        like( $Ret->decoded_content(), qr{<HasNext>\w+</HasNext>}, 'GetReportListByNextToken returns HasNext' );
    }
    else {
        return 'Skipped getReportListByNextToken - no NextToken to check.';
    }
}

sub getReportRequestCount : Test {
    my $self = shift;
    return if (1);    # skipped because getReportRequestCount is flakey.
    my $Ret = $self->{Remote}->getReportRequestCount();
    like( $Ret->decoded_content(), qr{<Count>\d+</Count>}, 'GetReportRequestCount returns a number' );
}

sub getReportRequestList : Test {
    my $self = shift;
    my $Ret  = $self->{Remote}->getReportRequestList();
    like( $Ret->decoded_content(), qr{<HasNext>\w+</HasNext>}, 'GetReportRequestList returns HasNext' );
}

sub requestReport : Test {
    my $self = shift;
    my $Ret = $self->{Remote}->requestReport( ReportType => '_GET_MERCHANT_CANCELLED_LISTINGS_DATA_' );
    like( $Ret->decoded_content(), qr{<ReportRequestId>\d+</ReportRequestId>}, 'RequestReport returns ReportRequestId' );
}

sub cancelReportRequests : Test {
    my $self = shift;
    my $Ret = $self->{Remote}->cancelReportRequests( 'ReportTypeList.Type.1' => '_GET_MERCHANT_CANCELLED_LISTINGS_DATA_' );
    like( $Ret->decoded_content(), qr{<Count>\d+</Count>}, 'CancelReportRequests returns a number' );
}

sub getReportRequestListByNextToken : Test {
    my $self = shift;
    my $Ret  = $self->{Remote}->getReportRequestList();
    if ( $Ret->decoded_content() =~ qr{<NextToken>([^<]+)</NextToken>}x ) {
        $Ret = $self->{Remote}->getReportRequestListByNextToken( NextToken => $1 );
        like( $Ret->decoded_content(), qr{<HasNext>\w+</HasNext>}, 'GetReportRequestListByNextToken returns HasNext' );
    }
    else {
        return 'Skipped getReportRequestListByNextToken - no NextToken to check.';
    }
}

sub manageReportSchedule_enable_hourly : Test(2) {
    my $self = shift;
    my $Ret = $self->{Remote}->manageReportSchedule( ReportType => '_GET_MERCHANT_CANCELLED_LISTINGS_DATA_', Schedule => '_1_HOUR_' );
    like( $Ret->decoded_content(), qr{<Schedule>_1_HOUR_</Schedule>}, 'ManageReportSchedule confirms hourly schedule.' );
    $Ret = $self->{Remote}->getReportScheduleList( 'ReportTypeList.Type.1' => '_GET_MERCHANT_CANCELLED_LISTINGS_DATA_' );
    like( $Ret->decoded_content(), qr{<Schedule>_1_HOUR_</Schedule>}, 'GetReportScheduleList confirms hourly schedule.' );
}

sub manageReportSchedule_disable : Test(2) {
    my $self = shift;
    my $Ret = $self->{Remote}->manageReportSchedule( ReportType => '_GET_MERCHANT_CANCELLED_LISTINGS_DATA_', Schedule => '_NEVER_' );
    like( $Ret->decoded_content(), qr{<Schedule>_NEVER_</Schedule>}, 'ManageReportSchedule confirms _NEVER_ schedule.' );
    $Ret = $self->{Remote}->getReportScheduleList( 'ReportTypeList.Type.1' => '_GET_MERCHANT_CANCELLED_LISTINGS_DATA_' );
    unlike( $Ret->decoded_content(), qr{<Schedule>_NEVER_</Schedule>}, 'GetReportScheduleList does not contain _NEVER_ schedule.' );
}

sub getOrderServiceStatus : Test {
    my $self = shift;
    my $Ret  = $self->{Remote}->getOrderServiceStatus();
    like( $Ret->decoded_content(), qr{<Status>[^<]+</Status>}x, 'GetServiceStatus returns status.' );
}

sub getReportScheduleListByNextToken_with_invalid_token : Test {
    my $self = shift;
    my $Ret = $self->{Remote}->getReportScheduleListByNextToken( NextToken => 'INVALID_TOKEN' );
    like( $Ret->decoded_content(), qr{<Code>NextTokenCorrupted</Code>}, 'GetReportScheduleListByNextToken with invalid token returns NextTokenCorrupted' );
}

sub getReportScheduleCount : Test {
    my $self = shift;
    my $Ret  = $self->{Remote}->getReportScheduleCount();
    like( $Ret->decoded_content(), qr{<Count>[^<]+</Count>}x, 'GetServiceStatus returns a number.' );
}

sub updateReportAcknowledgements_with_invalid_reportId : Test {
    my $self = shift;
    my $Ret = $self->{Remote}->updateReportAcknowledgements( 'ReportIdList.Id.1' => 1, 'Acknowledged' => 'true' );
    like( $Ret->decoded_content(), qr{<Code>InvalidReportId</Code>}, 'UpdateReportAcknowledgements returns InvalidReportId' );
}

sub submitFeed_valid_content : Test {
    my $self = shift;
    my $xml  = <<'EOL';
<?xml version="1.0"?>
<AmazonEnvelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xsi:noNamespaceSchemaLocation="amzn-envelope.xsd">
  <Header>
    <DocumentVersion>1.01</DocumentVersion>
      <MerchantIdentifier>$MerchantIdentifier</MerchantIdentifier>
    </Header>
  <MessageType>_POST_INVENTORY_AVAILABILITY_DATA_</MessageType>
<Message>
  <MessageID>1</MessageID>
  <OperationType>Update</OperationType>
  <Inventory>
    <SKU>NON_EXISTING_SKU</SKU>
    <Quantity>0</Quantity>
  </Inventory>
</Message>
</AmazonEnvelope>
EOL

    my $Ret = $self->{Remote}->submitFeed( 'FeedType' => '_POST_INVENTORY_AVAILABILITY_DATA_', 'rContent' => \$xml );
    like( $Ret->decoded_content(), qr{><FeedSubmissionId>[^<]+</FeedSubmissionId>}x, 'submitFeed returns FeedSubmissionId.' );
}

sub submitFeed_missing_content : Test {
    my $self = shift;
    eval {
        $self->{Remote}->submitFeed( 'FeedType' => '_POST_INVENTORY_AVAILABILITY_DATA_' );
        fail('should throw PARAMETER_MISSING');
    };
    if ( ExistsError() ) {
        is( GetError->code, 'PARAMETER_MISSING', 'caught PARAMETER_MISSING exception.' );
    }
}

sub getFeedSubmissionList : Test {
    my $self = shift;
    my $Ret  = $self->{Remote}->getFeedSubmissionList();
    like( $Ret->decoded_content(), qr{<HasNext>[^<]+</HasNext>}, 'getFeedSubmissionList returns HasNext.' );
}

sub getFeedSubmissionListByNextToken : Test {
    my $self = shift;
    my $Ret = $self->{Remote}->getFeedSubmissionListByNextToken( NextToken => 'INVALID_TOKEN' );
    like( $Ret->decoded_content(), qr{<Code>NextTokenCorrupted</Code>}, 'GetFeedSubmissionListByNextToken with invalid token returns NextTokenCorrupted' );
}

sub getFeedSubmissionCount : Test {
    my $self = shift;
    my $Ret  = $self->{Remote}->getFeedSubmissionCount();
    like( $Ret->decoded_content(), qr{<Count>[^<]+</Count>}x, 'GetFeedSubmissionCount returns a number.' );
}

sub cancelFeedSubmissions : Test {
    my $self = shift;
    my $Ret = $self->{Remote}->cancelFeedSubmissions( 'FeedSubmissionIdList.Id.1' => '1058369303' );
    like( $Ret->decoded_content(), qr{<Code>InvalidFeedSubmissionId</Code>}x, 'CancelFeedSubmissions returns InvalidFeedSubmissionId error.' );
}

sub getFeedSubmissionResult_with_invalid_FeedSumissionId : Test {
    my $self = shift;
    my $Ret = $self->{Remote}->getFeedSubmissionResult( 'FeedSubmissionId' => 'INVALID_ID' );
    like( $Ret->decoded_content(), qr{<Code>InvalidRequest</Code>}x, 'GetFeedSubmissionResult with invalid FeedSumissionId returns InvalidRequest error' );
}

sub fillStringToMod4Length_empty : Test {
    my $self = shift;
    my $str  = '';
    $self->{Remote}->_fillStringToMod4Length( \$str );    ## no critic (ProhibitPrivateSubroutineUse)
    is( $str, '', 'Empty string ok.' );
}

sub fillStringToMod4Length_1char : Test {
    my $self = shift;
    my $str  = 'x';
    $self->{Remote}->_fillStringToMod4Length( \$str );    ## no critic (ProhibitPrivateSubroutineUse)
    is( $str, 'x===', 'one char gets filled up.' );
}

sub fillStringToMod4Length_4char : Test {
    my $self = shift;
    my $str  = 'xxxx';
    $self->{Remote}->_fillStringToMod4Length( \$str );    ## no critic (ProhibitPrivateSubroutineUse)
    is( $str, 'xxxx', 'four char ok.' );
}

sub secondaryMarketplaceIDsToRequestArgs_empty : Test {
    my $self = shift;
    is_deeply(
        $self->{Remote}->_secondaryMarketplaceIDsToRequestArgs(),    ## no critic (ProhibitPrivateSubroutineUse)
        {}, 'empty marketplace ids'
    );
}

sub secondaryMarketplaceIDsToRequestArgs_filled : Test {
    my $self = shift;

    $self->{Remote}->setSecondaryMarketplaceIDs( [qw( secondary tertiary )] );
    my $Ret = $self->{Remote}->_secondaryMarketplaceIDsToRequestArgs();    ## no critic (ProhibitPrivateSubroutineUse)
    is_deeply( $Ret, { 'MarketplaceIdList.Id.2' => 'secondary', 'MarketplaceIdList.Id.3' => 'tertiary' }, 'filled marketplace ids' ) or diag($Ret);
}

package main;                                                              ## no critic (ProhibitMultiplePackages)
use DE_EPAGES::Core::API::Script qw ( RunScript );

RunScript(
    Sub => sub {
        Test::Class->runtests;
    }
);

1;
