#========================================================================================
# package      DE_EPAGES::Amazon::API::Object::SubmitFeedResponseHandlerTest
# state        public
#----------------------------------------------------------------------------------------
# description  Tests SubmitFeedResponseHandler
#========================================================================================
package DE_EPAGES::Amazon::t::SubmitFeedResponseHandlerTest;
use base Test::Class;

use strict;
use Test::More;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Amazon::API::MWS::SubmitFeedResponseHandler;

# setup methods are run before every test method.
sub setUp : Test(setup) {
    my $self = shift;
    $self->{Object} = DE_EPAGES::Amazon::API::MWS::SubmitFeedResponseHandler->new();
}

sub handleEmptyResult : Test {
    my $self = shift;
    eval {
        $self->{Object}->handle('');
        fail('should throw exception');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'InvalidXMLResponse','error on empty result.');
    }
}

sub handleNonXMLResult : Test {
    my $self = shift;
    eval {
        $self->{Object}->handle('asldkjalsdkj');
        fail('should throw exception');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'InvalidXMLResponse','error on arbitrary string');
    }
}

sub handleOtherXMLResult : Test {
    my $self = shift;
    eval {
        $self->{Object}->handle(<<'EOL');
<?xml version="1.0" ?>
<root></root>
EOL
        fail('should throw exception');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'InvalidXMLResponse','error on required field not found.');
    }
}

sub handleXMLHeaderOnlyResult : Test {
    my $self = shift;
    eval {
        $self->{Object}->handle(<<'EOL');
<?xml version="1.0" ?>
EOL
        fail('should throw exception');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code, 'InvalidXMLResponse','error on missing xml body.');
    }
}

sub handleExpectedXMLResult : Test {
    my $self = shift;
    $self->{Object}->handle(<<'EOL');
<?xml version="1.0"?>
<SubmitFeedResponse xmlns="http://mws.amazonaws.com/doc/2009-01-01/">
  <SubmitFeedResult>
    <FeedSubmissionInfo>
      <FeedSubmissionId>2291326430</FeedSubmissionId>
      <FeedType>_POST_PRODUCT_DATA_</FeedType>
      <SubmittedDate>2009-02-20T02:10:35+00:00</SubmittedDate>
      <FeedProcessingStatus>_SUBMITTED_</FeedProcessingStatus>
    </FeedSubmissionInfo>
  </SubmitFeedResult>
  <ResponseMetadata>
    <RequestId>75424a38-f333-4105-98f0-2aa9592d665c</RequestId>
  </ResponseMetadata>
</SubmitFeedResponse>
EOL

    is($self->{Object}->Result, '2291326430', 'extracted FeedSubmissionId');
}

package main;                                                                     ## no critic (ProhibitMultiplePackages)
use DE_EPAGES::Core::API::Script qw ( RunScript );

RunScript(
    Sub => sub {
        Test::Class->runtests;
    }
);

1;