use strict;

use DE_EPAGES::Amazon::API::SOAP::SOAPResponseHandler;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use SOAP::Lite;
use Test::More;

sub Main {
    testDoc();
    done_testing();
}

sub testDoc {
    ## no critic (RequireSpacesNotTabs ProhibitLongLines)
    my $data = <<'EOL';
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SE="http://schemas.xmlsoap.org/soap/encoding/"><SOAP-ENV:Body><SOAP-ENV:Fault><faultcode>SOAP-ENV:Server</faultcode><faultstring>_DOCUMENT_NO_LONGER_AVAILABLE_</faultstring><detail><ns0:ExceptionTypeName xmlns:ns0="WASP_ExceptionNs">WASP_Exception</ns0:ExceptionTypeName><ns1:ExceptionMessage xmlns:ns1="WASP_ExceptionNs">_DOCUMENT_NO_LONGER_AVAILABLE_</ns1:ExceptionMessage><ns2:ExceptionStackTrace xmlns:ns2="WASP_ExceptionNs">
	caught in 'void MIME::_WASP_MerchantInterfaceInstanceMethods::getDocument(MIME::MerchantInterfaceImpl*, WASP_MessageAttachmentManager*, WASP_DeserializationContext*, WASP_SerializationContext*)' at /opt/brazil-pkg-cache/packages/MerchantGatewayServer/MerchantGatewayServer-1.1.22.89/RHEL5/generic-flavor/src/waspcServer/merchantInterfaceMIMEImpl.cpp:609
	thrown in 'static void MIME::_WASP_MerchantInterfaceInstanceMethods::_WASP_getDocumentCallback(WASP_CallContext*, void*, WASP_Exception*, WASP_ExceptionEnv*)' at /opt/brazil-pkg-cache/packages/MerchantGatewayServer/MerchantGatewayServer-1.1.22.89/RHEL5/generic-flavor/src/waspcServer/merchantInterfaceMIMEImpl.cpp:627
	caught in 'void WASP_CallContext::callResponseCallbacks(WASP_Exception*)' at runtime/CallContext.cpp:184</ns2:ExceptionStackTrace></detail></SOAP-ENV:Fault></SOAP-ENV:Body></SOAP-ENV:Envelope>
EOL

    my $Parser = SOAP::Deserializer->new;
    my $SOM    = $Parser->deserialize($data);

    my $handler = DE_EPAGES::Amazon::API::SOAP::SOAPResponseHandler->new();
    eval {
        $handler->handle($SOM);
        fail('should throw error.');
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        is( $Error->code, 'SOAPFault', 'Error type: SOAPFault' );
        is( $Error->vars()->{Code}, '_DOCUMENT_NO_LONGER_AVAILABLE_', 'SOAP error code' );
    }
}

RunScript( Sub => \&Main );
