use strict;
use warnings FATAL => 'all';
no warnings 'uninitialized'; ## no critic(ProhibitNoWarnings)
use utf8;
use 5.010;
## no critic( RequireBlockMap, RequireBlockGrep )
use Test::More;
use Test::Exception;

# use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Billsafe::API::Request;

use FindBin qw($Bin);
use lib "$Bin";

BEGIN {
    *Billsafe::Request::  = *DE_EPAGES::Billsafe::API::Request::  ;
    *Billsafe::Account::  = *DE_EPAGES::Billsafe::API::Account::  ;
    *Billsafe::Article::  = *DE_EPAGES::Billsafe::API::Article::  ;
    *Billsafe::Customer:: = *DE_EPAGES::Billsafe::API::Customer:: ;
    *Billsafe::Order::    = *DE_EPAGES::Billsafe::API::Order::    ;
}

use constant ETD => 'expecting to die';
use constant ETL => 'expecting to live';

sub main {

# Test constructors.
my $rexreq = qr/required/;
my $rextyp = qr/type constraint/;
throws_ok { Billsafe::Account->epages } $rexreq;
throws_ok { Billsafe::Account->epages( merchant_id      =>   333 ) } $rexreq;
throws_ok { Billsafe::Account->epages( merchant_license =>   333 ) } $rexreq;
throws_ok { Billsafe::Account->epages( merchant_id      => undef,
                                       merchant_license => 'abc' ) } $rextyp;
throws_ok { Billsafe::Account->epages( merchant_license => undef,
                                       merchant_id      => 'abc' ) } $rextyp;
my $account = Billsafe::Account->epages_merchant;
isa_ok $account, 'Billsafe::Account';
# Trigger exception.
dies_ok { Billsafe::Request->new } 'no account, but required';
# Check account data.
is $account->application_signature, 'ea1cbfec7ae6a5b5b3340ac2297b3a7a',
    'signature correct';
ok $account->application_version, 'version exists';
# Construct valid object.
my $bs = Billsafe::Request->new( account => $account, logger => \&diag, silent => 1 );
isa_ok $bs, 'DE_EPAGES::Billsafe::API::Request';

# Billsafe::Customer, test constructor.
my $customer = Billsafe::Customer->new;
isa_ok $customer, 'Billsafe::Customer';
lives_ok { Billsafe::Customer->new( not_a_hash_ref => 42 ) } ETL;
lives_ok { Billsafe::Customer->new( {invalid_parm => 42} ) } ETL;

# gender
dies_ok { Billsafe::Customer->new( {gender => 'a'} ) } 'm/f only';
lives_ok { Billsafe::Customer->new( {gender => 'f'} ) } 'gender valid';
lives_ok { Billsafe::Customer->new(   gender => undef   ) } 'undef should be ignored';
lives_ok { Billsafe::Customer->new( { gender => undef } ) } 'undef should be ignored';
{
    my @customers;
    push @customers, Billsafe::Customer->new(   gender => undef   );
    push @customers, Billsafe::Customer->new( { gender => undef } );
    for ( @customers ) {
        ok ! defined $_->gender,  'undef gender ignored - method returns undef';
        ok ! exists $_->{gender}, 'undef gender ignored - key does not exist';
    }
}

# dateOfBirth
dies_ok { Billsafe::Customer->new( {dateOfBirth => 9999} ) } ETD;
lives_ok { Billsafe::Customer->new( {dateOfBirth => '1977-07-07'} ) }
    'dateOfBirth valid';
{
    my @customers;
    push @customers, Billsafe::Customer->new(   dateOfBirth => undef   );
    push @customers, Billsafe::Customer->new( { dateOfBirth => undef } );
    for ( @customers ) {
        ok ! defined $_->dateOfBirth,  'undef dateOfBirth ignored - method returns undef';
        ok ! exists $_->{dateOfBirth}, 'undef dateOfBirth ignored - key does not exist';
    }
}

# Billsafe::Order, test constructor.
TODO: {
todo_skip q(wait for when MooseX::StrictConstructor is available), 1;
dies_ok { Billsafe::Order->new( invalid_parm => 42 ) } ETD;
}
lives_ok { Billsafe::Order->new } 'empty order';
lives_ok { Billsafe::Order->new( number => 'ep12345678' ) }
    'order with number';
lives_ok { Billsafe::Order->new( require_total => 1 ) }
    'order with option require_total';
lives_ok { Billsafe::Order->new( articles => [] ) }
    'order with articles';

# Billsafe::Article, test constructor.
throws_ok sub { Billsafe::Article->new( {number   => 42} ) }, $rexreq;
throws_ok sub { Billsafe::Article->new( {quantity => 42} ) }, $rexreq;

done_testing;

}

# RunScript Sub => \&main;
# RunScript versteckt den eigentlichen Fehler:
#   account required at C:/epages/Perl/lib/Carp.pm line 44.
# Es sollte aber kommen:
#   account required at t\01_billsafe.t line 31
main;
