## no critic( ProhibitUnusedVariablesEnhanced )
use strict;
use warnings;
use DE_EPAGES::Billsafe::API::Article;
use Test::More;

# expected rounding implementation
sub round { int( 100 * $_[0] + .5 ) / 100 }

my @expected_nvp_keys = qw( name number type quantity tax );
my @expected_nvp_keys_netto  = sort 'netPrice',   @expected_nvp_keys;
my @expected_nvp_keys_brutto = sort 'grossPrice', @expected_nvp_keys;
# optional: description quantityShipped

my %args = (
    name        => 'Apfel',
    type        => 'goods',
    number      => 'xxx777',
    grossPrice  => 1.99,
    quantity    => 3,
    tax         => 19,
);

# XXX netPrice tests fehlen

{   my $art = DE_EPAGES::Billsafe::API::Article->new( %args );
    is $art->quantity, 3, 'Menge bei 3';

    my %nvp = $art->as_nvp;
    ok defined $nvp{$_}, "$_ defined" for sort keys %nvp;
    is $nvp{quantity}, 3, 'auch via as_nvp';

    my @keys = sort keys %nvp;
    is "@keys", "@expected_nvp_keys_brutto", 'NVP members';

    is $nvp{grossPrice}, $args{grossPrice}, 'grossPrice via as_nvp';
    ok ! exists $nvp{netPrice}, 'no netPrice';
#   diag explain \%nvp;
}

$args{quantity} = 7.75;
{   my $art = DE_EPAGES::Billsafe::API::Article->new( %args );
    is $art->quantity, 1, 'Menge auf 1 korrigiert';

    my $price_fixed = round 1.99 * 7.75; 
    is $art->grossPrice, $price_fixed, 'price fixed';

    my $name_fixed = "$args{name} (x $args{quantity})";
    is $art->name, $name_fixed, 'name fixed (including original quantity)';

    my %nvp = $art->as_nvp;
    ok defined $nvp{$_}, "$_ defined" for sort keys %nvp;
    is $nvp{quantity}, 1, 'auch via as_nvp';
    is $nvp{grossPrice}, $price_fixed, 'price fixed';
    is $nvp{name}, $name_fixed, 'name fixed';

    my @keys = sort keys %nvp;
    is "@keys", "@expected_nvp_keys_brutto", 'NVP members';

    is $nvp{grossPrice}, $price_fixed, 'grossPrice via as_nvp';
    ok ! exists $nvp{netPrice}, 'no netPrice';

    $art->quantity(3);
    is $art->quantity,      3,          'supplied quantity again';
    is $art->grossPrice,    1.99,       'supplied price again';
    is $art->name,          'Apfel',    'supplied name again';

    $art->quantity(3.33);
    is $art->quantity,                    1,    'fixed quantity again';
    is $art->grossPrice, round(1.99 * 3.33),    'fixed price again';
    is $art->name,         'Apfel (x 3.33)',    'fixed name again';
}

$args{quantityShipped} = 3.5;
{   my $art = DE_EPAGES::Billsafe::API::Article->new( %args );
    is $art->quantity, 1, 'Menge auf 1 korrigiert';
    is $art->quantityShipped, 1, 'Menge auf 1 korrigiert';

    my $price_fixed = round 1.99 * 7.75; 
    is $art->grossPrice, $price_fixed, 'price fixed';
}

done_testing;
