## no critic( RequireBlockMap )
use strict;
use warnings;
use DE_EPAGES::Billsafe::API::Order;
use DE_EPAGES::Billsafe::API::Article;
use Test::More;

my @expected_order_keys = qw(
order_amount order_currencyCode order_taxAmount product);
my @expected_order_keys_but_undef = qw/ order_number /;

my @expected_art_keys = qw( name number type quantity tax );
my @expected_art_keys_netto  = sort 'netPrice',   @expected_art_keys;
my @expected_art_keys_brutto = sort 'grossPrice', @expected_art_keys;
# optional: description quantityShipped

sub check_nvp {
    my( $order, %nvp ) = @_;
    # Build list of expected keys.
    my $num_articles = scalar $order->get_articles;
    my @expected_keys = (@expected_order_keys, @expected_order_keys_but_undef);
    for my $ak ( @expected_art_keys_brutto ) {
        push @expected_keys,
            map 'articleList_' . $_ . '_' . $ak, 0 .. $num_articles - 1;
    }
    # Got all expexted keys? With defined values?
    for ( @expected_keys ) {
        ok exists $nvp{$_}, "$_ exists";
        ok defined $nvp{$_}, "$_ defined"
            unless $_ ~~ \@expected_order_keys_but_undef;
    }
    # Any unexpected keys?
    for ( keys %nvp ) {
        ok $_ ~~ \@expected_keys, "in list of expected keys: $_"
            unless $_ =~ m/(description|netPrice)$/;
    }
}

my %args = (
    name        => 'Apfel',
    type        => 'goods',
    number      => 'xxx777',
    grossPrice  => 1.99,
    quantity    => 3,
    tax         => 19,
);

{   my $order = DE_EPAGES::Billsafe::API::Order->new;
    $order->add_article( DE_EPAGES::Billsafe::API::Article->new(%args) );
    @args{qw/name number grossPrice quantity/} = qw/Birne yyy888 2.49 7/;
    $order->add_article( DE_EPAGES::Billsafe::API::Article->new(%args) );
    my %nvp = $order->as_nvp;
    is scalar keys %nvp, 17, 'correct number of keys';
    check_nvp $order, %nvp;
    diag explain \%nvp;
    my $exp_amount = '23.40';
    is $order->amount,     $exp_amount, "amount $exp_amount";
    is $nvp{order_amount}, $exp_amount, "amount $exp_amount via NVP";
    is $nvp{order_currencyCode}, 'EUR', 'EUR';
}

done_testing;
