use strict;
use warnings FATAL => 'all';
no warnings 'uninitialized'; ## no critic(ProhibitNoWarnings)
use utf8;
use 5.010;
## no critic( DE_EPAGES::Style::RequireCorrectImports )
## no critic( RequireBlockMap, RequireBlockGrep )
use DE_EPAGES::Billsafe::API::Request;
use DE_EPAGES::Core::API::Script qw( RunScript );

use Test::More;
use Test::Exception;

use FindBin qw($Bin);
use lib "$Bin";

BEGIN {
    *Billsafe::Request::  = *DE_EPAGES::Billsafe::API::Request::  ;
    *Billsafe::Account::  = *DE_EPAGES::Billsafe::API::Account::  ;
    *Billsafe::Article::  = *DE_EPAGES::Billsafe::API::Article::  ;
    *Billsafe::Customer:: = *DE_EPAGES::Billsafe::API::Customer:: ;
    *Billsafe::Order::    = *DE_EPAGES::Billsafe::API::Order::    ;
}

use constant DIAG => 0;
use constant ORDER_WITH_PAYOUT => 'Milu1300876174';
use constant ORDER_WOUT_PAYOUT => 'Milu1300875577';
use constant OK => 'OK';

sub main {

my $account = Billsafe::Account->epages_merchant;
my $billsafe = Billsafe::Request->new( account => $account, logger => \&diag, silent => 1 );
isa_ok $billsafe, 'DE_EPAGES::Billsafe::API::Request';

my $sub;
my %map;
my $req;
my $rsp;
my %result;

$sub = sub {
    $billsafe->getPayoutStatus( # passing both should error out
        orderNumber => 'GanzEgal234', transactionId => 123456 )
};
dies_ok { %map = $sub->() } 'transactionId xor orderNumber';
dies_ok { $req = $sub->() } 'transactionId xor orderNumber (2)';

$sub = sub { $billsafe->getPayoutStatus( orderNumber => ORDER_WITH_PAYOUT ) };
%map = $sub->();
DIAG and diag explain \%map;
$req = $sub->();
isa_ok $req, 'HTTP::Request';
$rsp = $billsafe->request( $req );
isa_ok $rsp, 'HTTP::Response';
%result = $billsafe->parse_nvp( $rsp );
DIAG and diag explain \%result;
is $result{ack}, OK, OK;
ok exists $result{ $_ }, "got $_" for qw/
payoutList_0_amount
payoutList_0_date
payoutList_0_settlementNumber
returnList_0_amount
returnList_0_date
returnList_0_settlementNumber
/;

$sub = sub { $billsafe->getPayoutStatus( orderNumber => ORDER_WOUT_PAYOUT ) };
%map = $sub->();
DIAG and diag explain \%map;
$req = $sub->();
isa_ok $req, 'HTTP::Request';
$rsp = $billsafe->request( $req );
isa_ok $rsp, 'HTTP::Response';
%result = $billsafe->parse_nvp( $rsp );
DIAG and diag explain \%result;
is scalar keys %result, 1, 'only one key in reply';
is $result{ack}, OK, OK;

done_testing;
}

RunScript Sub => \&main;
