use strict;
use Test::More tests => 4;
use DE_EPAGES::Content::API::XML::StyleExportDriver qw( ExportStyle );
use DE_EPAGES::Content::API::XML::StyleImportHandler qw( ImportStyle );
use DE_EPAGES::Core::API::File qw( ExistsFile DeleteFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadObjectByPath ExistsObjectByPath );

my $ROOTPATH = '/Test';
my $SHOPALIAS = 'TestShop1';
my $SHOPPATH = "$ROOTPATH/$SHOPALIAS";

sub Main {
    my $Shop = LoadObjectByPath($SHOPPATH);

    my $Style = $Shop->get('DefaultStyle');
    my $FileName = 'exportStyle.zip';
    my $LanguageID = $Shop->get('LanguageID');
    my $MultiLanguageExport = 1;
    my $ImportStartPage = 1;

    DeleteFile( $FileName );
    ok( !ExistsFile( $FileName ), 'before export' );

    my $aWarnings = ExportStyle($Style, $FileName, $LanguageID, $MultiLanguageExport);
    note 'Warning: '.$_->shortMessage foreach @$aWarnings;
    ok( ExistsFile( $FileName ), 'after export' );

    my ($aImportWarnings, $ImportStyle) = ImportStyle($FileName, $Shop, $LanguageID, $ImportStartPage);
    note 'Warning: '.$_->shortMessage foreach @$aImportWarnings;
    ok( 0 == @$aImportWarnings, 'import' );
    isnt( $ImportStyle->alias, $Style->alias, 'import style alias' );
    DeleteFile( $FileName );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    if( ExistsObjectByPath($ROOTPATH) ) {
        note 'deleting shop';
        LoadObjectByPath($ROOTPATH)->delete
    }
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($SHOPPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    note 'creating shop';
    my $Folder = LoadRootObject()->folder('Test');
    $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );
    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        Init();
        Main();
        Cleanup();
    } );
});

