use strict;
use feature 'switch';
use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::Core::API::File qw (
    GetSubDirectories
    GetFileNames
    ExistsDirectory
);

sub Main {

    #files that dont have to be checked
    my $hExceptions = {
        'EbayRegions.xml' => 1,
        'Languages.xml' => 1,
        'DistributorShopTypes.xml' => 1,
        'DefaultShop.xml' => 1,
    };

    my $hStatistic = {};

    #check Cartridge xml files
    my $aCartridgeDirs = GetSubDirectories($ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES',{});

    #check every cartridge
    foreach my $CartridgeDir (@$aCartridgeDirs){
        #ignore exceptions
        next if $CartridgeDir =~ /\/CVS$/;

        #print "\ncheck Cartridge '$CartridgeDir'";

        if (ExistsDirectory($CartridgeDir . '/Database/XML')){

            #get *.xml files under ../Database/XML
            my $aXMLFiles = GetFileNames($CartridgeDir . '/Database/XML', {'recursive' => 1} );

            #check every file
            foreach my $XMLFile (@$aXMLFiles){

                #get file name from path
                my $FileName = $XMLFile;
                $FileName =~ s{
                    .*\/(
                    [\_0-9a-zA-Z\.]+    #file name
                    \.xml               #file type
                    )$
                }{$1}x;

                #ignore exceptions
                next unless $XMLFile =~ /.+\.xml$/;
                next if $XMLFile =~ /\/SQL\//;
                next if $XMLFile =~ /\/mysql\//;
                next if $hExceptions->{$FileName};
                next if $XMLFile =~ m{ShopChildDependencies\.xml$}x;

                if ($XMLFile =~ /Translation\.\w+\.\w\w\.xml$/x){
                    #make check for translation files
                    $hStatistic->{'CheckedFiles'}->{$XMLFile} = 1;
                    CheckTranslationFile($XMLFile,$hStatistic);
                }
                else {
                    #make check for normal xml file
                    $hStatistic->{'CheckedFiles'}->{$XMLFile} = 1;
                    CheckXMLFile($XMLFile,$hStatistic);
                }
            }
        }
    }

    #test the builded statistic
    TestHelptopics($hStatistic);

    ok(1, 'Tests run');
}

#========================================================================================
# function     CheckXMLFile
#----------------------------------------------------------------------------------------
# syntax       CheckXMLFile($XMLFile,$hStatistic);
#----------------------------------------------------------------------------------------
# description  Runs tests for a normal xml file.
#----------------------------------------------------------------------------------------
# input        $FilePath | path to xml file | string
# input        $hStatistic  | will filled with check Informations | ref.hash
#========================================================================================
sub CheckXMLFile {
    my ($FilePath, $hStatistic) = @_;

    #define exception tags
    my $hExceptions = {
        '' => 1,
        'epages' => 1,
        'Dependency' => 1,
    };

    #define tags to looking for
    my $hSearchClasses = {
        'HookFunction' => 1,
        'SiteLocale' => 1,
        'ShippingLevel' => 1,
        'FeatureCollectionFeature' => 1,
        'EbayTemplate' => 1,
        'EcoParticipationCategory' => 1,
        'SearchDefinition' => 1,
        'Provider' => 1,
        'Action' => 1,
        'ViewAction' => 1,
        'ChangeAction' => 1,
        'Hook' => 1,
        'TwengaMailType' => 1,
        'FormField' => 1,
        #'Dependency' => 1,
        'EbayEnvironment' => 1,
        'StyleGroup' => 1,
        'Permission' => 1,
        'AmazonSite' => 1,
        'Portal' => 1,
        'Group' => 1,
        #'AttributeValue' => 1,
        #'WSRoleMethod' => 1,
        'PaymentTypeLogo' => 1,
        'TaxArea' => 1,
        'Class' => 1,
        'TaxRate' => 1,
        'FeaturePack' => 1,
        'TermsAndConditions' => 1,
        'User' => 1,
        'NavBar' => 1,
        'ShopType' => 1,
        'Feature' => 1,
        'WebServiceMethod' => 1,
        'ProductType' => 1,
        'Link' => 1,
        'Form' => 1,
        'StartPage' => 1,
        #'PageTypeHelpTopicExt' => 1,
        'PaymentType' => 1,
        'StyleGroupMap' => 1,
        'Imprint' => 1,
        'Role' => 1,
        'MailType' => 1,
        'Style' => 1,
        'PortalSite' => 1,
        #'Attribute' => 1,
        'Category' => 1,
        'MediaGallery' => 1,
        'UnitOfMeasurement' => 1,
        'SearchParameter' => 1,
        'NavElement' => 1,
        #'EbayPaymentMethod' => 1,
        #'TaxClass' => 1,
        #'ContentChildClass' => 1,
        'TemplateType' => 1,
        #'Menu' => 1,
        #'FormLoop' => 1,
        #'BusinessHours' => 1,
        #'Locale' => 1,
        #'EbayShippingLocation' => 1,
        #'StyleTemplateVariation' => 1,
        #'Currency' => 1,
        #'SearchSort' => 1,
        #'TemplateTypeFolder' => 1,
        #'ProductFolder' => 1,
        #'MailTypeTemplate' => 1,
        #'ProviderStore' => 1,
        #'TaxAreaCountry' => 1,
        #'StyleTemplate' => 1,
        #'ProductTypeFolder' => 1,
        #'NavBarElement' => 1,
        'HelpTopic' => 1,
        'ViewAction' => 1,
        'PageType' => 1,
        'RoleAction' => 1,
        #'EbayCallType' => 1,
        #'PageTypeHelpTopic' => 1,
        'Object' => 1,
        #'WebService' => 1,
        #'EbayShippingMethod' => 1,
        #'NavElementGroup' => 1,
        #'PrivacyPolicy' => 1,
        #'Language' => 1,
        #'SearchFrom' => 1,
        'PaymentLogo' => 1,
        #'Template' => 1,
        'ShippingType' => 1,
        #'GroupMember' => 1,
        #'Country' => 1,
        #'TaxMatrix' => 1,
        #'TwengaSite' => 1
    };

    #generate xml structure from file
    my $XMLStructImporter = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $XMLStruct = $XMLStructImporter->getXMLStructure($FilePath, {'distinguish_multiple_appearence' => 1});

    #complete expected delete-Attributes
    AddDeleteIfParentHasDelete($XMLStruct, {'File' => $FilePath} );

    #check every element in xml structure
    foreach my $Element (@{$XMLStruct->getElements()}){

        #ignore exceptions
        next if $hExceptions->{$Element};

        #get class of element in xml structure
        my $Class = $XMLStruct->getXMLStructAttribute($Element, 'Class');

        #fill visited classes to statistic
        $hStatistic->{'CheckedClasses'}->{$Class} = 1;
        $hStatistic->{'CheckedTagNames'}->{$XMLStruct->getXMLStructAttribute($Element, 'TagName')} = 1;

        #check if class is a potential target
        next unless $hSearchClasses->{$Class};

        #check if element has an delete or a reference attribute
        if (
            not(
                $XMLStruct->existsTagAttribute($Element, 'reference') or
                $XMLStruct->existsTagAttribute($Element, 'delete')
            )
        ){
            ok(0, "file '$FilePath' element '$Element' delete or reference does not exists");
        }
        elsif($XMLStruct->existsTagAttribute($Element, 'delete')){

            ##get cleaned tag path
            #my $TagPath = _GetNameFreeFromAppearence($Element);

            ##check if element is defined in an other file
            #if ( exists $hStatistic->{'XMLTagDefinition'}->{$TagPath} ){
            #    #element already defined in an other file
            #    if ($FilePath ne $hStatistic->{'XMLTagDefinition'}->{$TagPath} ){
            #        #different file create failed test message
            #        ok(0, "multiple definition of element '$Element' in file '$FilePath' and " . $hStatistic->{'XMLTagDefinition'}->{$TagPath});
            #    }
            #}

            ##set file where element is defined
            #$hStatistic->{'XMLTagDefinition'}->{$TagPath} = $FilePath;
        }

        #check for additional statistic
        given($Element){
            when (IsHelpTopic($XMLStruct, $_)){#element is a helptopic
                RegisterHelpTopic(
                    _GetNameFreeFromAppearence($Element),
                    $hStatistic,
                    $FilePath,
                    $XMLStruct
                );
            }
        }
    }
}

#========================================================================================
# function     CheckTranslationFile
#----------------------------------------------------------------------------------------
# syntax       CheckTranslationFile($XMLFile,$hStatistic);
#----------------------------------------------------------------------------------------
# description  Runs tests for a translation xml file.
#----------------------------------------------------------------------------------------
# input        $FilePath | path to xml file | string
# input        $hStatistic  | will filled with check Informations | ref.hash
#========================================================================================
sub CheckTranslationFile{
    my ($FilePath, $hStatistic) = @_;

    #generate xml structure from file
    my $XMLStructImporter = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $XMLStruct = $XMLStructImporter->getXMLStructure($FilePath, {'distinguish_multiple_appearence' => 1});

    #check every element in xml structure
    foreach my $Element (@{$XMLStruct->getElements()}){
        given($Element){
            when (IsHelpTopic($XMLStruct,$_)){#element is a helptopic
                RegisterHelpTopic(
                    _GetNameFreeFromAppearence($Element),
                    $hStatistic,
                    $FilePath,
                    $XMLStruct
                );
            }
            when (IsHelpTopicAttribute($_)){#element is a helptopic attribute
                RegisterHelpTopicAttribute(
                    _GetNameFreeFromAppearence($Element),
                    $hStatistic,
                    $FilePath,
                    $XMLStruct
                );
            }
            default{
                #normal adding $element
                #roadworks
            }
        }
    }
}

#========================================================================================
# function     AddDeleteIfParentHasDelete
#----------------------------------------------------------------------------------------
# syntax       AddDeleteIfParentHasDelete($XMLStruct, $hInfo);
#----------------------------------------------------------------------------------------
# description  Adds delete attribute for every element in xml structure that have a parent with delete attribute.
#----------------------------------------------------------------------------------------
# input        $XMLStruct | xml structure object | object
# input        $hInfo  | contains information for recursive calling | ref.hash
#========================================================================================
sub AddDeleteIfParentHasDelete {
    my ($XMLStruct, $hInfo) = @_;

    #for every element in xml structure
    foreach my $Element (@{$XMLStruct->getElements()}){
        #ignore element that already have a delete attribute
        if ($XMLStruct->existsTagAttribute($Element, 'delete')){
            #if (ParentWithDeleteExists($XMLStruct, $Element)){#parent have a delete attribute and subobject have a redundant delete
            #    ok(
            #        0,
            #        "file '" . $hInfo->{'File'} . "' referenced element '$Element' has a redundant delete attribute in tag"
            #    );
            #}
            next;
        };

        if (
            $XMLStruct->getTagAttribute($Element, 'reference') and
            IsFirstAppearence($Element)
        ){#first tag appearence and referenced
            if (ParentWithDeleteExists($XMLStruct, $Element)){#parent have a delete attribute
                ok(
                    0,
                    'file \'' . $hInfo->{'File'} . "' referenced element '$Element' has no parent tag with delete"
                );
            }
        }
        elsif (not $XMLStruct->existsTagAttribute($Element, 'delete')){
            #call recursive function
            ParentWithDeleteExists($XMLStruct,$Element,{'setdeletes' => 1});
        }
    }
}

#========================================================================================
# function     IsFirstAppearence
#----------------------------------------------------------------------------------------
# syntax       $IsFirst = IsFirstAppearence($Element);
#----------------------------------------------------------------------------------------
# description  Check element name if no appearence string is contained.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
# return       $IsFirst | true if this is the first appearence of the object | boolean
#========================================================================================
sub IsFirstAppearence {
    my ($Element) = @_;

    return not $Element =~ m{
        \:appearance  #signal of multiple appearance of tag
        \(\d+\)$            #appearance counter at the ending of element string
    }x;
}

#========================================================================================
# function     IsHelpTopic
#----------------------------------------------------------------------------------------
# syntax       IsHelpTopic($Element);
#----------------------------------------------------------------------------------------
# description  Check element name if belong to a HelpTopic.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
#========================================================================================
sub IsHelpTopic{
    my ($XMLStruct, $Element) = @_;

    return 1 if ($XMLStruct->getXMLStructAttribute($Element,'TagName') eq 'HelpTopic');
    return 0;
}

#========================================================================================
# function     IsHelpTopicAttribute
#----------------------------------------------------------------------------------------
# syntax       IsHelpTopicAttribute($Element);
#----------------------------------------------------------------------------------------
# description  Check element name if belong to a HelpTopic attribute.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
#========================================================================================
sub IsHelpTopicAttribute{
    my ($Element) = @_;

    return $Element =~ m{
        \/HelpTopics\/         #Search Text on beginning
        [a-zA-Z\_\-0-9\.]+      #Helptopic name
        \/AttributeValue\{      # ../AttributeValue{
        Language\-\>.+\;        #Language->...;
        Name\-\>.+\}$           #Name->...} at the end
    }x;
}

#========================================================================================
# function     _GetNameFreeFromAppearence
#----------------------------------------------------------------------------------------
# syntax       _GetNameFreeFromAppearence($Element);
#----------------------------------------------------------------------------------------
# description  Returns element name free from appearence strings.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
# return       $CleanElement | element of xml structure object free from appearence strings| string
#========================================================================================
sub _GetNameFreeFromAppearence{
    my ($Element) = @_;

    $Element =~ s{
        \:appearance    #apearence signal
        \(\d+\)         #appearance counter
    }{}xg;

    return $Element;
}

#========================================================================================
# function     ParentWithDeleteExists
#----------------------------------------------------------------------------------------
# syntax       ParentWithDeleteExists($XMLStruct, $Element, $hOptions);
#----------------------------------------------------------------------------------------
# description  Returns true if parent element with delete attribute exists.
#----------------------------------------------------------------------------------------
# input        $XMLStruct | xml structure object | object
# input        $Element | element of xml structure object | string
# input        $hOptions  | contains information for recursive calling | ref.hash
# return       $ParentExists | true if parent with delete attribute exists | bool
#========================================================================================
sub ParentWithDeleteExists {
    my ($XMLStruct, $Element, $hOptions) = @_;

    #no parent with delete exists
    return 0 if $Element eq '';

    #get parent element
    my $Parent = $XMLStruct->parentElement($Element);

    if ($XMLStruct->existsTagAttribute($Parent, 'delete')){#direkt parent has delete attribute
        $XMLStruct->setTagAttribute($Element, 'delete', 1) if $hOptions->{'setdeletes'};
        return 1;
    }
    else{#direkt parent has no delete attribute, have to go on recursive
        return ParentWithDeleteExists($XMLStruct, $Parent, $hOptions);
    }
}

#========================================================================================
# function     RegisterHelpTopic
#----------------------------------------------------------------------------------------
# syntax       RegisterHelpTopic($Element, $hStatistic, $FilePath, $XMLStruct);
#----------------------------------------------------------------------------------------
# description  Register HelpTopic in statistic.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
# input        $hStatistic  | will filled with check Informations | ref.hash
# input        $FilePath | path to xml file | string
# input        $XMLStruct | xml structure object | object
#========================================================================================
sub RegisterHelpTopic{
    my ($Element, $hStatistic, $FilePath, $XMLStruct) = @_;

    if ($XMLStruct->existsTagAttribute($Element, 'delete')){
        $hStatistic->{'HelpTopics'}->{$Element}->{'DefinedAt'}->{$FilePath} = 1;
    }
    elsif ($XMLStruct->existsTagAttribute($Element, 'reference')){
        $hStatistic->{'HelpTopics'}->{$Element}->{'ReferencedAt'}->{$FilePath} = 1;
    }
    else{
        $hStatistic->{'HelpTopics'}->{$Element}->{'ReferencedAt'}->{$FilePath} = 1;
    }
}

#========================================================================================
# function     RegisterHelpTopicAttribute
#----------------------------------------------------------------------------------------
# syntax       RegisterHelpTopicAttribute($Element, $hStatistic, $FilePath, $XMLStruct);
#----------------------------------------------------------------------------------------
# description  Register HelpTopic attribute in statistic.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
# input        $hStatistic  | will filled with check Informations | ref.hash
# input        $FilePath | path to xml file | string
# input        $XMLStruct | xml structure object | object
#========================================================================================
sub RegisterHelpTopicAttribute{
    my ($Element, $hStatistic, $FilePath, $XMLStruct) = @_;

    my $HelpTopic = $XMLStruct->parentElement($Element);

    $hStatistic->{'HelpTopics'}->{$HelpTopic}->{'AttrDefinedAt'}->{$FilePath} = 1;
}

#========================================================================================
# function     TestHelptopics
#----------------------------------------------------------------------------------------
# syntax       TestHelptopics($hStatistic);
#----------------------------------------------------------------------------------------
# description  Test for HelpTopics in statistic.
#----------------------------------------------------------------------------------------
# input        $hStatistic  | will filled with check Informations | ref.hash
#========================================================================================
sub TestHelptopics{
    my ($hStatistic) = @_;

    foreach my $HelpTopic (keys %{$hStatistic->{'HelpTopics'}}){
        if (scalar (keys %{$hStatistic->{'HelpTopics'}->{$HelpTopic}->{'AttrDefinedAt'}}) < 1){
            ok(0, "HelpTopic $HelpTopic has no translation attributes");
        }
    }
}

RunScript(
    'Sub' => \&Main,
);

done_testing();

1;