use strict;
use Test::More tests => 10;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::UPSWorldShip::API::XML::UPSPackingSlipExportDriver qw(XMLExportPackingSlip);
use DE_EPAGES::UPSWorldShip::DAL::UPSWorldShip qw (DAL_GetInfoUPSWorldShipTrackingsByTrackingNo);
use DE_EPAGES::Core::API::File qw ( GetFileContent DeleteFile);
use Text::Diff;

my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();

    Init();
    # create order
    my $Order = LoadObjectByPath($ROOTPATH.'/TestShop/1001/Orders/1001');
    ok( defined $Order, "order exists");

    # create ups packing slip
    my $UPSPackingSlip = $Order->path('Documents/1001-1');
    ok( defined $UPSPackingSlip, "ups packing slip exists");

    # save ups tracking number
    $UPSPackingSlip->set({'TrackingNo'=>'0123456789'});
    ok( $UPSPackingSlip->get('TrackingNo') eq '0123456789', "saving ups tracking number" );
    ok( (scalar @{DAL_GetInfoUPSWorldShipTrackingsByTrackingNo('0123456789')}), "tracking number exists");

    # delete ups tracking number
    $UPSPackingSlip->set({'TrackingNo'=>undef});
    ok( (not defined $UPSPackingSlip->get('TrackingNo')), "deleting ups tracking number" );
    my $a = DAL_GetInfoUPSWorldShipTrackingsByTrackingNo('0123456789');
    ok( (not scalar $a->[0]), "tracking number doesn't exist");

    # export ups packing slip
    XMLExportPackingSlip('export.xml', $UPSPackingSlip, 'de_DE');
    DiffPlainFile();
    DeleteFile("export.xml");

    Cleanup();
    DisconnectSite();
    return;
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    ok( 1, 'cleanup' );
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('initial.xml', $ROOTPATH);
    ok( 1, 'init' );
    return 1;
}

sub DiffPlainFile() {
    my $rContent1 = GetFileContent("export.exp");
    my $rContent2 = GetFileContent("export.xml");
    ok($$rContent1 eq $$rContent2, 'export ups packing slip') or diag(diff($rContent1, $rContent2));
}

RunScript(Sub=>\&Main, Debug=>1);
