use strict;
use Test::More;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::WebInterface::API::UserAgent qw( IsUserMobilePhone );

sub CheckDevices {
    my ($hParams) = @_;
##no critic (ProhibitLongLines RequireInterpolationOfMetachars)
    # mobile phones
    my @Mobiles = (
        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16 AppEngine-Google; (+http://code.google.com/appengine; appi',
        'Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543a Safari/419.3 (botmobi find.mobi/bot.html find@mtld.mobi)/',
        'Opera/9.80 (J2ME/MIDP; Opera Mini/5.0 (iPhone; U; CPU iPhone 0S 3.0 like Mac 0S X; en-us; compatible; Googlebot/18.794; U; US) Presto/2.4.15',
        'Browser Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) ASUS;GALAXY6',
        'Mozilla/4.0 (compatible; MSIE 7.0; Windows Phone OS 7.0; Trident/3.1; IEMobile/7.0) Microsoft Corporation;CEPC',
        'Nokia6820/2.0 (4.83) Profile/MIDP-1.0 Configuration/CLDC-1.0 (compatible; Googlebot-Mobile/2.1; +http://www.google.com/bot.html)',
        'Mozila/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Geckto) Version/3.0 Mobile/3A101a Safari/419.3',
        'MIDP',
        'BlackBerry8800/4.5 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/102',
        'Mozilla/5.0 (Linux; U; Android 1.6; en-gb; HTC Magic Build/DRC92) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1',
        'IEMobile',
        'Maemo',
        'Mozilla/5.0 (webOS/1.0; U; en-US) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/1.0 Safari/525.27.1 Pre/1.0',
        'bingbot_mobile',
        'msnbot_mobile',
        'Googlebot-Mobile',
        'Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30', # google nexus S
        'Mozilla/5.0 (Linux; U; Android 2.1-update1; en-gb; HTC Desire Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17', # htc desire 3,7" phone
        'Mozilla/5.0 (Linux; U; Android 2.1-update1; de-de; HTC Desire 1.19.161.5 Build/ERE27) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17', # htc desire 3,7" phone
        'Mozilla/5.0 (Linux; U; Android 2.2; en-dk; Desire_A8181 Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',   # htc desire 3,7" phone
        'Mozilla/5.0 (Linux; U; en-US) AppleWebKit/528.5+ (KHTML, like Gecko, Safari/528.5+) Version/4.0 Kindle/3.0 (screen 600x800; rotate)',   # kindle reader
    );
    if($hParams->{'test'}) {
        ok( IsUserMobilePhone($_), $_ . ' is mobile') foreach @Mobiles;
    } else {
        IsUserMobilePhone($_) foreach @Mobiles;
    }

    # usual browsers
    my @Browsers = (
        'Lynx/2.8.4rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.6c',
        'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)',
        'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; OfficeLiveConnector.1.3; OfficeLivePatch.0.0; .NET CLR 3.5.30729; .NET CLR 3.0.30618)',
        'Mozilla/4.77 [en] (X11; I; IRIX;64 6.5 IP30)',
        'Mozilla/5.0 (compatible; Konqueror/3.2; Linux 2.6.2) (KHTML, like Gecko)',
        'Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/125.2 (KHTML, like Gecko) Safari/125.8',
        'Mozilla/5.0 (OS/2; U; Warp 4.5; de; rv:1.8.1.11) Gecko/20071129 PmWFx/2.0.0.11',
        'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13',
        'Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.9.0.10) Gecko/2009042316 Firefox/3.0.10',
        'Mozilla/5.0 (X11; U; Linux i586; en-US; rv:1.7.3) Gecko/20040924 Epiphany/1.4.4 (Ubuntu)',
        'Opera/9.80 (Macintosh; Intel Mac OS X; U; en) Presto/2.2.15 Version/10.00',
        'msnbot',
        'bingbot',
        'googlebot',
    );
    if($hParams->{'test'}) {
        ok( !IsUserMobilePhone($_), $_ . ' is desktop') foreach @Browsers;
    } else {
        !IsUserMobilePhone($_) foreach @Browsers;
    }

    # tablets are non-mobiles too
    my @Tablets = (
        'Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI39) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',  # Xoom tablet
        'Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13', # Xoom tablet
        'Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; Xoom Build/HWI69) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13', # Xoom tablet
        'Mozilla/5.0 (Linux; U; Android 3.2; en-us; sdk Build/1.6.0_209) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1', # sdk tablet
        'Mozilla/5.0 (Linux; U; Android 3.1; de-de; AT100 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13', # toshiba at100
        'Mozilla/5.0 (Linux; U; Android 3.0.1; en-us; A500 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13', # 10,1" acer a500
        'Mozilla/5.0 (Linux; U; Android 3.0; xx-xx; A500 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13', # 10,1" acer a500
        'Mozilla/5.0 (Linux; U; Android 3.2; xx-xx; A200 Build/HTK55D) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13', # 10,1" Acer A200 (Picasso_E)
        'Mozilla/5.0 (Linux; U; Android 3.0; xx-xx; A100 Build/HRI66) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13', # 7" Acer Iconia Tab A100
        'Mozilla/5.0 (Linux; U; Android 3.1; xx-xx; GT-P7310 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13', # 8,9" samsung tablet
        'Mozilla/5.0 (Linux; U; Android 3.1; xx-xx; Sony Tablet S Build/THMAS10000) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13', # 9,4" Sony Tablet S
        'Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; BNTV250 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1', # 7" Barnes and Noble NOOK Tablet
        'Mozilla/5.0 (Linux; U; Android 2.1; xx-xx; A101IT Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2', # 10" archos tablet
        'Mozilla/5.0 (Linux; U; Android 2.1; xx-xx; MID7015 Build/ECLAIR) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2', # 7" Coby Kyros
        'Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; MID1125 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 10" Coby tablet
        'Mozilla/5.0 (Linux; U; Android 2.2; ru-; ViewPad7 Build/FRG83) AppleWebKit/525.10+ (KHTML, like Gecko) Version/3.0.4 Mobile Safari/523.12.2', # 7" ViewSonic ViewPad7
        'Mozilla/5.0 (Linux; U; Android 3.0; en-us; SCH-I800 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 7" Samsung SCH-I800
        'Mozilla/5.0 (Linux; U; Android 2.2; en-us; SCH-I800 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 7" Samsung SCH-I800
        'Mozilla/5.0 (Linux; U; Android 2.2; en-gb; SAMSUNG GT-P1000 Tablet Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 7" Samsung GT-P1000
        'Mozilla/5.0 (Linux; U; Android 2.2; en-us; GT-P1000 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 7" Samsung GT-P1000
        'Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000R Build/FROYO) Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M Build/FROYO)', # 7" Samsung GT-P1000
        'Mozilla/5.0 (Linux; U; Android 2.2; en-ca; GT-P1000M Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 7" Samsung GT-P1000
        'Mozilla/5.0 (Linux; U; Android 2.2; en-us; SGH-T849 Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 7" tablet
        'Mozilla/5.0 (Linux; U; Android 2.2; en-us; SHW-M180S Build/FROYO) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 7" tablet
        'Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 7" Kindle fire
        'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; en-us; Silk/1.1.0-80) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16 Silk-Accelerated=true', # 7" Kindle fire
        'Mozilla/5.0 (Linux; U; Android 2.3; xx-xx; Kindle Fire Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1', # 7" Kindle fire
        # from strato logfiles
        'Mozilla/5.0 (Linux; U; Android 3.1; de-de; GT-P7510 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',
        'Mozilla/5.0 (Linux; U; Android 3.2; de-de; LIFETAB_P9514 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',
    );
    if($hParams->{'test'}) {
        ok( !IsUserMobilePhone($_), $_ . ' is tablet') foreach @Tablets;
    } else {
        !IsUserMobilePhone($_)  foreach @Tablets;
    }
}

RunScript( Sub => sub {
    CheckDevices({ 'test' => 1});

    my $Iters = 2000;
    my $time1 = time;
    for(0 .. $Iters) {
        CheckDevices({ 'test' => 0});
    }
    cmp_ok( (time - $time1) / $Iters, '<', 0.01, 'Mobile check needs less than 0.01 sec');

    done_testing();
});
