## no critic( RequireCorrectImports )
## no critic( RequireBlockMap )
use utf8;
use 5.012;
use DE_EPAGES::Address::API::Attributes::Address_Ext;
use DE_EPAGES::Billsafe::UI::BasketOrder;
use Test::More;
use Test::MockObject;

BEGIN { *Bubo:: = *DE_EPAGES::Billsafe::UI::BasketOrder:: }

BEGIN {
    *Billsafe::Request::  = *DE_EPAGES::Billsafe::API::Request::  ;
    *Billsafe::Account::  = *DE_EPAGES::Billsafe::API::Account::  ;
    *Billsafe::Article::  = *DE_EPAGES::Billsafe::API::Article::  ;
    *Billsafe::Customer:: = *DE_EPAGES::Billsafe::API::Customer:: ;
    *Billsafe::Order::    = *DE_EPAGES::Billsafe::API::Order::    ;
}

# What is the purpose of this file?
# Testing the troublesome makeCustomer() subroutine.

sub make_mock {
    my( $street ) = @_;
    my $mock = Test::MockObject->new;
    $mock->mock( get => sub {
        my( $self, $keys ) = @_;
        my %tbl;
        @tbl{ @$keys } = map "I am $_", @$keys;
        $tbl{'Street'}   = $street      if ('Street' ~~ @$keys);
        $tbl{'Street2'}  = undef        if ('Street2' ~~ @$keys);
        $tbl{'Birthday'} = undef        if ('Birthday' ~~ @$keys);
        $tbl{'Gender'}   = time % 2 + 1 if ('Gender' ~~ @$keys);

        if ('StreetOnly' ~~ @$keys) {
            $tbl{'StreetOnly'} = (DE_EPAGES::Address::API::Attributes::Address_Ext::_ExtractHouseNumber($street))[0];## no critic( ProtectPrivateSubs )
        }
        if ('HouseNumber' ~~ @$keys) {
            $tbl{'HouseNumber'} = (DE_EPAGES::Address::API::Attributes::Address_Ext::_ExtractHouseNumber($street))[1];## no critic( ProtectPrivateSubs )
        }

        return \%tbl;
    });
    return $mock;
}

my @test_data = (
    [ 'Gartenstraße 23b' => 'Gartenstraße', '23b' ],
    [ 'Gürkchengasse drei' => 'Gürkchengasse drei', '' ],
    [ '33 rue du cauchemar' => 'rue du cauchemar', '33' ],
    [ 'rue des courgettes' => 'rue des courgettes', '' ],
    [ '1a avenue de Waterloo' => 'avenue de Waterloo', '1a' ],
);

for ( @test_data ) {
    my( $in, $exp_s, $exp_hn ) = @$_;

    my $mock = make_mock $in;

=pod
my @addrkeys = qw/
    ID Company Gender FirstName LastName Birthday
    Street Street2 Zipcode City EMail Phone/;
diag explain $mock->get( \@addrkeys );
diag explain $mock->get( [qw/eins zwei drei/] );
diag explain $mock;
=cut

    my $o = Bubo::makeCustomer( $mock );
    isa_ok $o, 'Billsafe::Customer' or die explain $o;
    # diag explain $o;
    is $o->houseNumber, $exp_hn, "houseNumber: $exp_hn";
    is $o->street, $exp_s, "street: $exp_s";
}

done_testing;
