#!perl -w
use Test::More;
use strict;
use DE_EPAGES::Core::API::Html qw(
    BalanceHTMLTags
    RemoveHtmlTags
);

plan tests => 10;
# BalanceHTMLTags
my $SourceText  = '<p>blubb';
my $GoalText    = '<html><head></head><body><p>blubb</p></body></html>';
cmp_ok(BalanceHTMLTags($SourceText, {'getCompleteDocument' => 1}), 'eq', $GoalText, 'BalanceHTMLTags make snippet to document');

$SourceText  = '<html><head><body att=sdsd><p>blubb';
$GoalText    = '<html><head></head><body att="sdsd"><p>blubb</p></body></html>';
cmp_ok(BalanceHTMLTags($SourceText, {'getCompleteDocument' => 1}), 'eq', $GoalText, 'BalanceHTMLTags corerct document');

$SourceText  = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
$GoalText    = '<p></p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags headings and paragraphs');

$SourceText  = '<p><table><tr><td>test</tr><tr><td><div><strong><!-- pagebreak -->';
$GoalText    = '<p></p><table><tr><td>test</td></tr><tr><td><div><strong><!-- pagebreak --></strong></div></td></tr></table>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags malformed table');

$SourceText  = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf<table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s</p><p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig<table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak -->';
$GoalText    = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf</p><table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s<p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig</a></span></i></p><table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak --></td></tr></tbody></table>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags complex from tinymce');

$SourceText = '<p class="test><span style="test2"></p>';
$GoalText   = '<p class="test&gt;&lt;span style=" test2"="test2&quot;"></p>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags missing quotes at the end');

$SourceText = '<p class="test test2 id="test><span style="test2"></p>';
$GoalText   = '<p class="test test2 id=" test="test"><span style="test2"></span></p>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags missing quotes with more values');

$SourceText = '<p class=test"><span style="test2"></p>';
$GoalText   = '<p class="test&quot;"><span style="test2"></span></p>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags missing quotes at the beginning');

$SourceText = '<script type="text/javascript">jQuery("<script>")</script>';
$GoalText   = '<script type="text/javascript">jQuery("<script>")</script>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags with script tags');

# RemoveHtmlTags
$SourceText  = <<'END_TEXT';
    <p>
      <h1> <span color="#333311">headline</span> <strong><!-- pagebreak --></strong></h1>
    <p>text1<br>text 2<hr />&nbsp;&nbsp;text3 &amp;</p>
    <script language="JScript">
      document.all.obsolete = 1;
    </script>
    <STYLE>
      document.all.obsolete = 1;
    </STYLE>
    fin.
END_TEXT
$GoalText = <<'END_TEXT';
headline
text1
text 2
  text3 &
fin.
END_TEXT
is( RemoveHtmlTags($SourceText)."\n", $GoalText, 'RemoveHtmlTags' );
