use strict;
use Test::More tests => 1;
use XML::LibXSLT;
use XML::LibXML;
use DE_EPAGES::Core::API::Script qw( RunScript );

RunScript( Sub => sub {

    my $parser = XML::LibXML->new();
    my $doc = $parser->parse_file('camels.xml');
    my $style_doc = $parser->parse_file('camels.xsl');

    my $xslt = XML::LibXSLT->new();
    my $stylesheet = $xslt->parse_stylesheet($style_doc);
    my $results = $stylesheet->transform($doc);
    my $HTML = $stylesheet->output_string($results);

    my $expectedHTML = <<'END_HTML';
<html xmlns:fo="http://www.w3.org/1999/XSL/Format">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<title>Know Your Dromedaries</title>
</head>
<body><table bgcolor="#eeeeee" border="1">
<tr>
<th>Species</th>
<th>No of Humps</th>
<th>Disposition</th>
</tr>
<tr>
<td>Camel</td>
<td>1 or 2</td>
<td>Cranky</td>
</tr>
<tr>
<td>Llama</td>
<td>1 (sort of)</td>
<td>Aloof</td>
</tr>
<tr>
<td>Alpaca</td>
<td>(see Llama)</td>
<td>Friendly</td>
</tr>
</table></body>
</html>
END_HTML

    cmp_ok( $HTML, 'eq', $expectedHTML, 'camels.xml + camels.xsl -> camels.html' );
});
