use Test::More tests => 3;
use strict;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::DocdataPay::API::Object::RemoteDocdataPay;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

sub testTransaction {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            SKIP: {
                my $Remote = DE_EPAGES::DocdataPay::API::Object::RemoteDocdataPay->new(
                    'ServiceURL' => 'https://test.docdatapayments.com/ps/com.tripledeal.paymentservice.servlets.PaymentService',
                    'IsLogging'  => 1,
                );
                
                $Remote->newPaymentCluster({
                     'merchant_name'           => 'epages_com',
                     'merchant_password'       => 'AbG2987',
                     'merchant_transaction_id' => time(),
                     'profile'                 => 'standard',
                     'client_id'               => '1001',
                     'price'                   => 102.15,
                     'cur_price'               => 'EUR',
                     'client_email'            => 'o.trosien@epages.com',
                     'client_firstname'        => 'Oliver',
                     'client_lastname'         => 'Trosien',
                     'client_address'          => 'Pilatuspool 2',
                     'client_zip'              => '20355',
                     'client_city'             => 'Hamburg',
                     'client_country'          => 'DE',
                     'client_language'         => 'en',
                     'description'             => '1x Campingaz Trister',
                     'days_pay_period'         => 14
                });
                
                ok(defined $Remote->paymentClusterID(), 'transaction id: '.$Remote->paymentClusterID());
                ok(defined $Remote->paymentClusterKey(),'transaction key: '.$Remote->paymentClusterKey());
                
                my $TransStatus = $Remote->statusPaymentCluster({
                    'payment_cluster_key' => $Remote->paymentClusterKey(),
                    'merchant_name'       => 'epages_com',
                    'merchant_password'   => 'AbG2987',
                });
                
                ok(ElementOf($TransStatus, qw (NN NY YN YY)), 'Status: '. $TransStatus);
            };
        }
    );
}


sub Main {
    SKIP: {
      skip( 'Skip tests as long as reliability of test cases remains unclear.', 3 );
      testTransaction;
    }
    return;
}

RunScript( Sub => \&Main );

1;