use strict;
use Test::Simple tests => 7;

use Getopt::Long;
use DE_EPAGES::Installer::API::CartridgesXMLFilePatcher;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Object::API::XMLCompare qw ( CompareXML );
use DE_EPAGES::Core::API::File qw ( DeleteFile CopyFile);
use DE_EPAGES::Object::API::XMLStructureImportHandler;

sub Main {

    local $| = 1;

    my $DeleteFiles = 0;
    my $Help;

    GetOptions(
        'help'          => \$Help,
        'deletefiles'  => \$DeleteFiles,
    );

    usage() if $Help;

    my $time1 = time;

    my $hOptions = {'checkdelete' => '1', 'checkreference' => '1', 'printmessage' => '1'};

    #test 01 - Permissions
    my $CartridgesXMLFile = 'Cartridges_orig.xml';
    my $DeleteFile = 'del_Cartridges.xml';
    my $ImportFile = 'imp_Cartridges.xml';
    my $CartridgesXMLResultFile = 'Cartridges_result.xml';

    CopyFile($CartridgesXMLFile, $CartridgesXMLResultFile);

    my $FilePatcher=DE_EPAGES::Installer::API::CartridgesXMLFilePatcher->new($CartridgesXMLResultFile);
    my $XMLStructImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();

    $FilePatcher->deleteElements($DeleteFile);
    ok (CompareXML($CartridgesXMLResultFile, 'Cartridges_del.xml', $hOptions), 'delete 1st');
    ok (CompareXML('Cartridges_del.xml', $CartridgesXMLResultFile, $hOptions), 'delete 2nd');

    DeleteFile($CartridgesXMLResultFile) if ($DeleteFiles);

    CopyFile($CartridgesXMLFile, $CartridgesXMLResultFile);

    $FilePatcher->importElements($ImportFile);
    ok (CompareXML($CartridgesXMLResultFile, 'Cartridges_imp.xml', $hOptions), 'import 1st');
    ok (CompareXML('Cartridges_imp.xml', $CartridgesXMLResultFile, $hOptions), 'import 2nd');

    #check for positions
    my $XMLStruct = $XMLStructImportHandler->getXMLStructure($CartridgesXMLResultFile, {'XML-Type' => 'epages_cartridges_xml'});

    #get positions
    my $BeforeInsertPosition = $XMLStruct->getXMLStructAttribute('/DatabaseType{Name->Store}/Cartridge{Package->DE_EPAGES::ExternalShipping}','Position');
    my $InsertPosition = $XMLStruct->getXMLStructAttribute('/DatabaseType{Name->Store}/Cartridge{Package->DE_EPAGES::MRW}','Position');

    ok(
        ($XMLStruct->existsElement('/DatabaseType{Name->Store}/Cartridge{Package->DE_EPAGES::MRW}') and
        ($BeforeInsertPosition == $InsertPosition -1 )),
        'correct position of MRW Cartridge in Store'
    );

    $BeforeInsertPosition = $XMLStruct->getXMLStructAttribute('/DatabaseType{Name->Fantasy}/Cartridge{Package->DE_EPAGES::ExternalShipping}','Position');
    $InsertPosition = $XMLStruct->getXMLStructAttribute('/DatabaseType{Name->Fantasy}/Cartridge{Package->DE_EPAGES::MRW}','Position');

    ok(
        ($XMLStruct->existsElement('/DatabaseType{Name->Fantasy}/Cartridge{Package->DE_EPAGES::MRW}') and
        ($BeforeInsertPosition == $InsertPosition -1 )),
        'correct position of MRW Cartridge in Fantasy'
    );

    $BeforeInsertPosition = $XMLStruct->getXMLStructAttribute('/DatabaseType{Name->Fantasy3}/Cartridge{Package->DE_EPAGES::ExternalShipping}','Position');
    $InsertPosition = $XMLStruct->getXMLStructAttribute('/DatabaseType{Name->Fantasy3}/Cartridge{Package->DE_EPAGES::MRW}','Position');

    ok(
        ($XMLStruct->existsElement('/DatabaseType{Name->Fantasy3}/Cartridge{Package->DE_EPAGES::MRW}') and
        ($BeforeInsertPosition == $InsertPosition -1 )),
        'correct position of MRW Cartridge in Fantasy3'
    );


    DeleteFile($CartridgesXMLResultFile) if ($DeleteFiles);

    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;

}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags] datafiles

flags:
    -help          show the command line options
    -deletefiles   delete generated patch files

Example:
    perl $0 -deletefiles
END_USAGE
    exit 2;
}

RunScript( Message => 'long', Sub => \&Main);

1;
