use Test::Simple tests => 11;

use strict;
use DE_EPAGES::Core::API::File qw( GetFileContent WriteFile );
use DE_EPAGES::Core::API::Script qw( RunScript );

use DE_EPAGES::Mail::API::Mail;
use DE_EPAGES::Mail::API::SMTP;

use utf8; # all text in this file is supposed to be UTF-8

RunScript(
    Sub => \&Main
);

## no critic qw(ProhibitLongLines)
sub Main {

    my $Smtp = DE_EPAGES::Mail::API::SMTP->new( 'mail', 25 ); ## no critic qw(ProhibitMagicNumbers)

    $Smtp->connection( sub {
        my $MailObject;
        $MailObject = CreateObject();
        ok(SendPlainTextMail($MailObject), 'plain text');
        ok(SendMain($MailObject, $Smtp));

        $MailObject = CreateObject();
        ok(SendPlainTextwithAttachments($MailObject), 'plain text with attachment');
        ok(SendMain($MailObject, $Smtp));

        $MailObject = CreateObject();
        ok(SendPlainHTMLMail($MailObject), 'HTML mail without images');
        ok(SendMain($MailObject, $Smtp));

        $MailObject = CreateObject();
        ok(SendPlainHTMLMailwithPictures($MailObject), 'HTML mail with images');
        ok(SendMain($MailObject, $Smtp));

        $MailObject = CreateObject();
        ok(SendPlainHTMLMailwithPicturesAndAttachments($MailObject), 'HTML mail without images and attachments');
        ok(SendMain($MailObject, $Smtp));
    });

    # test convertHtml2Plain
    my $Html = ${ GetFileContent('ProductRecommendation.html') };
    my $Plain = ${ GetFileContent('ProductRecommendation.txt') };
    my $Result = DE_EPAGES::Mail::API::Mail->convertHtml2Plain( $Html );
    WriteFile( 'ProductRecommendation.result', \$Result );
    ok( $Result eq $Plain, 'convertHtml2Plain' );
}

#--------------------------------------------------------------------------------------------------


sub CreateObject {

    # neues mail object
    my $Mail = DE_EPAGES::Mail::API::Mail->new(
        {'HEADER_ENC'=> 'iso-8859-1',                     #(optional) (default: utf-8)
        'SUBJECT'    => 'Testing the email cartridge ÄÖÜß', #(optional)
         'FROM'      => 'Milestones <Milestones@epages.de>', #(mandatory)
        'NOTIFY'     => 1,                                #(optional)
         #'REPLY'    => 'Milestones@epages.de',           #(optional)
        'TO'         => [ 'm.mustermann@epages.de' ],     #(mandatory)
        'TO_SkipBad' => 0,                                #(optional) skip void e-mail addresses
         #'CC'       => ['Bäcker Bärbel <bbecker@epages.de>'], #(optional)
         #'BCC'      => ['fred@epages.de'],               #(optional)
        });

    return  $Mail;
}


### sample for a plain text Mail
sub SendPlainTextMail {
    my ($Mail) = @_;

    $Mail->addText({'content'   => 'Das ist Plain Text',# mandatory
                        'encoding'  => 'quoted-printable',  # optional (default: quoted-printable)
                        'type'       => 'text/plain',       # optional (default: text/plain)
                        'type.charset'=> 'utf-8',           # optional (default: utf-8)
                        }
                      );

    return 1;
}


### sample for a plain text Mail with attachments
sub SendPlainTextwithAttachments {
    my ($Mail) = @_;

    $Mail->addText({'content'   => 'Das ist Plain Text mit attachment',# mandatory
                        'encoding'  => 'quoted-printable',  # optional (default: quoted-printable)
                        'type'       => 'text/plain',       # optional (default: text/plain)
                        'type.charset'=> 'utf-8',           # optional (default: utf-8)
                        }
                      );

    # you can call this fkt with a list of filenames or call this fkt for each file
    $Mail->addAttachment([{'filename'    => 'Image2.jpg',  # mandatory: the filename to the Attachment
                               'type'        => 'image/jpg',                # optional (default: Application/Octet-stream )
                              },
                              {'filename'    => 'Mail.t',
                               'type'        => 'Application/Octet-stream',
                              }
                              ]
                            );

    return 1;
}

### sample for sending html mail
sub SendPlainHTMLMail {
    my ($Mail) = @_;


    $Mail->addHTML({'content'       => '<body bgcolor="gray"><h1>Hello HTML</h1></body>', #the html code (mandatory)
                        'encoding'      => 'quoted-printable',      # optional (default: quoted-printable) the html encoding
                        'type'          => 'text/html',             # optional (default: text/html)  the content type
                        'type.charset'  => 'utf-8',                 # optional (default: utf-8 ) the charset for html
                        'html2plain'    => 1,                       # optional (default: 0) if set to 1 alternate text is generated from html
                        'alttext'       => 'Alternate Text',        # optional  the alternate text if no convert is selected
                        'altencoding'   => 'quoted-printable',      # optional (default: quoted-printable) the alternate text encoding
                        'alt.type.charset' => 'utf-8',               # optional (default: utf-8) the charset for the alternate Text !ignored if html2plain!
                       }
        );

    return 1;
}

### sample for sending html mail with inline pictures
sub SendPlainHTMLMailwithPictures {
    my ($Mail) = @_;


    # the html content
    $Mail->addHTML({'content'        => '<body bgcolor="gray">'.
                            '<h1>HTML+Image Here\'s <i>my '.
                            '</i> image:</h1>'.
                            '<img src="cid:__franziska__" alt="Unser Bildchen!"></body>', #the html code (mandatory)
                        'encoding'       => 'quoted-printable',      # optional (default: quoted-printable) the html encoding
                        'type'           => 'text/html',             # optional (default: text/html)  the content type
                        'type.charset'   => 'utf-8',                 # optional (default: utf-8 ) the charset for html
                        'html2plain'     => 1,                       # optional (default: 0) if set to 1 alternate text is generated from html
                        'alttext'        => 'Alternate Text',        # optional  the alternate text if no convert is selected
                        'altencoding'    => 'quoted-printable',      # optional (default: quoted-printable) the alternate text encoding
                        'alt.type.charset' => 'utf-8',               # optional (default: utf-8) the charset for the alternate Text !ignored if html2plain!
                       }
        );

    # inline picture
    # you can add more than one  HTML attachment by calling this fkt again or add the file to the array
    $Mail->addHTMLAttachment([{'filename'    => 'Image2.jpg',  #mandatory the filename to the Attachment
                                 'includeid' => '__franziska__',    #mandatory the ID to find this include from the HTML code (<img src="cid:Image2">)
                                 'type'      => 'image/jpg',        #optional (default: Application/Octet-stream ) the content type
                                 },
                                 #{'filename'   => '<Path>',
                                 # 'includeid'  => '<ID>',
                                 # 'type'       => '<Type>',
                                 #}
                                 ]
        );

    return 1;
}

## sample or sending html+image and extra attachments
sub SendPlainHTMLMailwithPicturesAndAttachments {
    my ($Mail) = @_;

    # the html content
    $Mail->addHTML({'content'        => '<body bgcolor="gray">'.
                            '<h1>HTML+Image+Attachment:Here\'s <i>my '.
                            '</i> image:</h1>'.
                            '<img src="cid:__franziska__" alt="Unser Bildchen!"></body>', #the html code (mandatory)
                        'encoding'       => 'quoted-printable',      # optional (default: quoted-printable) the html encoding
                        'type'           => 'text/html',             # optional (default: text/html)  the content type
                        'type.charset'   => 'utf-8',                 # optional (default: utf-8 ) the charset for html
                        'html2plain'     => 1,                       # optional (default: 0) if set to 1 alternate text is generated from html
                        'alttext'        => 'Alternate Text',        # optional  the alternate text if no convert is selected
                        'altencoding'    => 'quoted-printable',      # optional (default: quoted-printable) the alternate text encoding
                        'alt.type.charset' => 'utf-8',               # optional (default: utf-8) the charset for the alternate Text !ignored if html2plain!
                       }
        );

    # inline picture
    # you can add more than one  HTML attachment by calling this fkt again or add the file to the array
    $Mail->addHTMLAttachment([{'filename'    => 'Image2.jpg',  #mandatory the filename to the Attachment
                                 'includeid' => '__franziska__',    #mandatory the ID to find this include from the HTML code (<img src="cid:Image2">)
                                 'type'      => 'image/jpg',        #optional (default: Application/Octet-stream ) the content type
                                 },
                                 #{'filename'   => '<Path>',
                                 # 'includeid'  => '<ID>',
                                 # 'type'       => '<Type>',
                                 #}
                                 ]
        );

    # you can call this fkt with a list of filenames or call this fkt for each file
    $Mail->addAttachment([{'filename'    => 'Image2.jpg',  # mandatory: the filename to the Attachment
                               'type'        => 'image/jpg',                # optional (default: Application/Octet-stream )
                              },
                              {'filename'    => 'Mail.t',
                               'type'        => 'Application/Octet-stream',
                              }
                              ]
        );

    return 1;
}

sub SendMain {
    my ($Mail, $Connection) = @_;

    $Mail->send( $Connection );
    return 1;
}

#Construction
#    MIME-Lite-HTML use a MIME-Lite object, and RFC2257 construction:


#    If html+images and text and attachments are present, construction use is:

#      -> multipart/mixed
#      --> each attachment
#      --> multipart/alternative
#      ------> text/plain
#      ------> multipart/related
#      -------------> text/html
#      -------------> each images

#    If html+images and text are present, construction use is:
#      --> multipart/alternative
#      ------> text/plain
#      ------> multipart/related
#      -------------> text/html
#      -------------> each images

#    If no images but text is present, this is that:
#      ---> multipart/alternative
#      -------> text/plain if present
#      -------> text/html

#    If images but no text, this is:
#      ---> multipart/related
#      -------> text/html
#      -------> each images

#    If no images and no text, this is:
#      ---> text/html
#
