#!perl -w
use strict;
use Test::More tests => 8;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw (RunOnStore GetTestSite);
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath );

sub Main {
    ## no critic qw(ProhibitMagicNumbers)
    ok( Init(), 'initialization' );

    my $Hour = LoadObjectByPath( '/Units/second/hour' );
    is( $Hour->get('Factor'), 3600, 'seconds per hour' );
    is( $Hour->durationUnitPlural, 'hours', 'duration unit of hour' );

    my $Night = LoadObjectByPath( '/Units/second/night' );
    is( $Night->get('Factor'), 86400, 'seconds per night' );
    is( $Night->durationUnitPlural, 'days', 'duration unit of night' );

    my $Box = LoadObjectByPath( '/Units/piece/box' );
    eval {
        is( $Box->durationUnitPlural, 'days', 'duration unit of box' );
    };
    ok( ExistsError(), 'Box is not a time unit' );
    is( GetError()->code(), 'NoTimeUnit', 'error code' );


    ok( Cleanup(), 'Cleanup' );
}

sub Cleanup {
    return 1;
}

sub Init {
    Cleanup();

    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
