dojo.provide('epages.cartridges.de_epages.presentation.widget.InlineEditor');
//dojo.require("dijit._Widget");
//dojo.require("dijit._Templated");
dojo.require("epages.widget.LocalizedWidget");
dojo.require("epages.widget");
dojo.require("epages.html");
dojo.require("epages.html.undo");
dojo.require("epages.event.domdispatcher");

// wrapper widget for all inline editors
dojo.declare(
	"epages.cartridges.de_epages.presentation.widget.InlineEditor",
	//[dijit._Widget, dijit._Templated],
	epages.widget.LocalizedWidget,
	{
		// from fragment
		value:          '',		// insert in displayNode
		valueFormatted:	'',		// insert in contentNode (textarea)
		inputName:      '',
		undoWidget:     '',
		title:          '',

		// public properties
		// widget or tinymce instance
		object:         null,
		templateString: '<div class="InlineEditor"><div dojoAttachPoint="displayNode" dojoAttachEvent="onclick:onClick,onmouseover:onMouseOver,onmouseout:onMouseOut">${!value}</div><textarea style="display: none" dojoAttachPoint="contentNode" id="InlineEditor_${id}" name="${inputName}">${!valueFormatted}</textarea></div>',
		translationName: dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget','templates/translation'),
		objectId:       null,
		attribute:      null,
		languageId:     null,
		undoObject:     null,
		isTyping:       false,
		isInited:       false,
		allowedNodes:   {},
		widgetsInTemplate: true,
		navbar:         '', // parent navbar eg. Content, Left, Top

		// is this widget always visible? (no show/hide switching)
		permanent:      false,

		// private properties
		_jsonIo:        null,
		_computedStyle: null,
		_relevantStyleAttributes: [
			"fontWeight",
			"fontFamily",
			"fontSize",
			"fontStyle",
			"color",
			"backgroundImage",
			"backgroundRepeat",
			"backgroundColor",
			"backgroundPosition",
			"marginTop",
			"marginBottom",
			"marginLeft",
			"marginRight",
			"paddingTop",
			"paddingBottom",
			"paddingLeft",
			"paddingRight",
			"maxHeight",
			"maxWidth"
			],
		// dimensions of the original source node
		_height:        null,
		_width:         null,
		_titleNode:     null,
		_resizeClone:   null,
		_placeHolderText: '...',

		postMixInProperties: function() {
			this.valueFormatted = this.value;	// default -> no formatting
			this.inherited("postMixInProperties", arguments);
		},

		create: function(params, srcNodeRef) {
			switch (srcNodeRef.nodeName.toLowerCase()) {
				case "input":
				case "textarea":
				case "select":
					this.value = srcNodeRef.value;
					break;
				case "img":
					this.value = srcNodeRef.src;
					break;
				default:
					this.value = srcNodeRef.innerHTML;
					break;
			}

			// copy style
			var tmpStyle = dojo.getComputedStyle(srcNodeRef);
			this._computedStyle = {};

			dojo.forEach(this._relevantStyleAttributes, function(prop) {
				var t = typeof tmpStyle[prop];
				if ((t == 'number' || t == 'string') && tmpStyle[prop] != "normal") {
					//alert(prop + ": " + tmpStyle[prop]);
					this._computedStyle[prop] = tmpStyle[prop];
				}
			}, this);

			// insert inputfield
			this.inputName = srcNodeRef.getAttribute('ep_input');
			if (this.inputName == null) {
				throw new Error("missing attribute ep_input for InlineEditor " + this.id);
			}
			this._height = srcNodeRef.scrollHeight;
			this._width = srcNodeRef.scrollWidth;
			var inputNameParts = this.inputName.split(':');
			this.objectId = inputNameParts[0];
			this.attribute = inputNameParts[1];
			this.languageId = inputNameParts[2];

			// call parent create method
			this.inherited('create', arguments);

			// insert title node to DOM
			this._titleNode = this.domNode.ownerDocument.createElement('span');
			if (this.title != '') {
				this._titleNode.innerHTML = this.title;
				dojo.addClass(this._titleNode, "TitleDiv");
				this._titleNode.style.display = 'none';
				srcNodeRef.title = '';
			}
			//this.domNode.insertBefore(this._titleNode, this.domNode.firstChild);
			this.domNode.parentNode.insertBefore(this._titleNode, this.domNode);
		},

		postCreate: function() {
			this._copyStyle(this.displayNode, this._computedStyle);
			this.inherited('postCreate', arguments);

			this.domNode.title = this.translate('InsertContentHere');

			if (this.permanent){
				this.show();
			}
			if (this.undoWidget != ''){
				this.undoObject = $$(this.undoWidget).getUndoObject();
			}
			//add content to a helper div to remove html tags for a correct empty check
			var testDiv = document.createElement('div');
			testDiv.innerHTML = this.contentNode.value;
			if (testDiv.textContent.length == 0 && this.displayNode){
				this.displayNode.innerHTML = this._placeHolderText;
			}
		},

		onClick: function(evt) {
			epages.topDojo.publish('InlineEditor/click', [{ instance: this, event: evt }]);
			this.show();
			dojo.stopEvent(evt);
		},

		onBlur: function(/*DOMEvent*/evt) {
			// description:
			//  method to override, called when editor may blur
			//  evt:
			//    event that triggered the blur
			// returns:
			//    whether to blur the editor or not
			return true; //Boolean
		},

		onMouseOver: function(evt) {
			$E(this.displayNode).addClassName('Highlighted');
		},

		onMouseOut: function(evt) {
			$E(this.displayNode).removeClassName('Highlighted');
		},

		show: function() {
			if (epages.cartridges.de_epages.presentation.widget.InlineEditor.selectedEditor != null &&
					epages.cartridges.de_epages.presentation.widget.InlineEditor.selectedEditor != this) {
				// close another opened editor
				epages.cartridges.de_epages.presentation.widget.InlineEditor.selectedEditor.hide();
			}

			this._titleNode.style.display = '';
			this.connect(this.contentNode, 'onchange', 'onChangeContentNode');
			// manage global InlineEditor instance
			epages.cartridges.de_epages.presentation.widget.InlineEditor.selectedEditor = this;
			epages.topDojo.publish('InlineEditor/show', [this]);
			if(this.domNode.clientWidth == 0) {
				this.domNode.style.minWidth = Math.max(0, this._width - this.domNode.offsetWidth) + "px";		// set content minwidth
			}
		},

		_copyStyle: function(node, style) {
			// summary: Copy the style from the source
			// description: -- taken from dijit.InlineEditBox --
			//              Don't copy ALL properties though, just the necessary/applicable ones

			dojo.forEach(this._relevantStyleAttributes, function(prop) {
				node.style[prop] = (style[prop] || "");
			}, this);

		},

		_resizeToContent: function() {
			// this timeout avoids some flickering
			window.setTimeout(dojo.hitch(this, function() {
				var node = this.contentNode;
				var clone = this._resizeClone;
				if (clone == null) {
					var clone = node.ownerDocument.createElement('div');
					var cs = dojo.getComputedStyle(node);
					var s = clone.style;
					for (var p in cs) {
						try {
							s[p] = cs[p];
						} catch(e) {
						}
					}
					// insert it into dom but dont show it
					clone.style.width = '100%';
					clone.style.height = 'auto';
					clone.style.overflow = 'visible';
					clone.style.position = 'absolute';
					clone.style.top = '-9999px';
					clone.style.whiteSpace = 'normal';
					node.parentNode.insertBefore(clone, node);
					this._resizeClone = clone;
				}
				if (node.value != '') {
					clone.innerHTML = epages.html.encodeEntity(node.value, true);
				} else {
					clone.innerHTML = '&nbsp;';
				}
				if (node.value.match(/[\r\n]$/)){
					clone.innerHTML += '&nbsp;';
				}
				var innerHeight = clone.scrollHeight;
				var offsetHeight = 0;
				node.style.height = (innerHeight + offsetHeight) + 'px';

				// callback for height correction in text fields
				if(this.correctCallback !== undefined){
					this.correctCallback(node);
				}
			}), 1);
		},

		hide: function(evt) {
			this._titleNode.style.display = 'none';
			epages.cartridges.de_epages.presentation.widget.InlineEditor.selectedEditor = null;
			if (this.contentNode.value == ''){
				this.displayNode.innerHTML = this._placeHolderText;
			}
			epages.topDojo.publish('InlineEditor/hide', [this]);
		},

		onChange: function() {
			// summary: write contents back to hidden input
			if(this.object){
				this.contentNode.value = this.object.getContent();
			}
			this.onChangeContentNode();
		},

		onChangeContentNode: function() {
			//summary: gets called, when data is changed (from outside)
			if(this.undoObject){
				this.undoObject.addInput(this.contentNode);
			}
		},

		getRemoteValue: function(/*Array.string*/additionalAttributes) {
		  // summary: get the correct value from the server, the content may be manipulated by the browser

			// create new json instance
			if (this._jsonIo == null){
				this._jsonIo = new epages.io.Json();
			}
			var attributes = [ this.attribute ];

			// add additional needed attributes
			if (additionalAttributes != null) {
				attributes = attributes.merge(additionalAttributes);
			}

			// get correct content from server, content may be modified by browser
			var value = this._jsonIo.loadSync('?', {
				'ViewObjectID': this.objectId,
				'ViewAction': 'JSONGet',
				'Attributes': attributes.join(',')
			});

			return value;
		}

	}
);

epages.cartridges.de_epages.presentation.widget.InlineEditor.selectedEditor = null;
$D.add( 'click',
	// summary: adds click event handler to document of editor and topWindow document
	// description: we need to check on click if we are still inside the editor
	//    or ouside. if the click was outside the hide methode of the current editor is called

	function() {
		//summary: return selected editor if available
		if (epages.cartridges.de_epages.presentation.widget.InlineEditor.selectedEditor != null){
			return epages.cartridges.de_epages.presentation.widget.InlineEditor.selectedEditor.domNode;
		}
		else{
			return null;
		}
	},
	function(evt) {
		var ed = epages.cartridges.de_epages.presentation.widget.InlineEditor.selectedEditor;
		if((ed && ed.contentNode == evt.target) //check if editor was clicked for Gecko and WebKit
		|| (ed && ed.contentNode == evt.srcElement)){ // check if editor was clicked for MSIE
			return;
		}

		if (ed != null && ed.onBlur(evt)){ //if blur event of editor was
			ed.hide();
		}
	},
	// outside
	true,
	[document, epages.topWindow.document]
);
